# -*- coding: utf-8 -*-
#
# This is the main application controls
# and the renaming functions.
#

"""
Copyright (C) 2006-2009 ianaré sévi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ianare@gmail.com
"""

# needed modules:
import wx
import sys
import platform
import os
import codecs
import re
import time
import wx.lib.buttons as buttons
import wx.lib.dialogs
import EnhancedStatusBar as ESB
import gettext
import locale
import getopt

# notebok windows:
import sorting
import errors

# dialogs:
import preferences
import helpDiag
import about
import langSelect
import editDialog

# utilities
import utils
import shutil1 # some earlier python versions have a bug in shutil


def create(parent):
    return MainWindow(parent)


[wxID_MAIN_WINDOW, wxID_MAIN_WINDOWDISPLAY, wxID_MAIN_WINDOWGO,
wxID_MAIN_WINDOWNOTEBOOK, wxID_MAIN_WINDOWPREVIEW,
wxID_MAIN_WINDOWSTATUSBAR1, wxID_MAIN_WINDOWSTATUSIMAGE,
wxID_MAIN_WINDOWAUTOPREVIEW, wxID_SELECTIONAREA,
wxID_BOTTOM_WINDOW, wxID_MAIN_WINDOWTHUMBSIZE,
wxID_MAIN_WINDOWIMGPREVIEW, wxID_MAIN_WINDOWUNDOREDO,

wxID_MAIN_WINDOWMENUHELPABOUT, wxID_MAIN_WINDOWMENUHELPHELP,
wxID_MAIN_WINDOWMENUHELPEXAMPLES, wxID_MAIN_WINDOWMENUHELPREHELP,
wxID_MAIN_WINDOWMENUHELPFORMATHELP,

wxID_MAIN_WINDOWMENUFILEEXIT, wxID_MAIN_WINDOWMENUFILEPREVIEW,
wxID_MAIN_WINDOWMENUFILEGO, wxID_MAIN_WINDOWMENUFILELOADINI,
wxID_MAIN_WINDOWMENUFILESAVEINI, wxID_MAIN_WINDOWMENUFILESAVELOG,

wxID_MAIN_WINDOWMENUPICKERALL, wxID_MAIN_WINDOWMENUPICKERNONE,
wxID_MAIN_WINDOWMENUPICKERWALK, wxID_MAIN_WINDOWMENUPICKERBROWSE,
wxID_MAIN_WINDOWMENUPICKEROK,

wxID_MAIN_WINDOWMENUEDITPREFERENCES, wxID_MAIN_WINDOWMENUFILEIMPORT,
wxID_MAIN_WINDOWMENUEDITLANG,

wxID_MENUEDIT, wxID_MENUREMOVE,

] = [wx.NewId() for _init_coll_menuEdit_Items in range(34)]

# ------------ the bottom preview virtual listcrtl:
class ListCtrl(wx.ListCtrl):
    def __init__(self, parent, ID, pos=wx.DefaultPosition,
                 size=wx.DefaultSize):
        style = wx.LC_REPORT|wx.LC_VIRTUAL

        self.parent = parent.GetGrandParent()

        wx.ListCtrl.__init__(self, parent, ID, pos, size, style)

        self.InsertColumn(col=0, format=wx.LIST_FORMAT_LEFT,
              heading=_(u"Location"), width=225)
        self.InsertColumn(col=1, format=wx.LIST_FORMAT_LEFT,
              heading=_(u"Original Name"), width=245)
        self.InsertColumn(col=2, format=wx.LIST_FORMAT_LEFT,
              heading=_(u"New Name"), width=265)

        self.SetMinSize(wx.Size(-1,110))

        self.SetItemCount(0)
        self.showDirs = False
        self.mode = 'preview'

        self.green = wx.ListItemAttr()
        self.green.SetBackgroundColour('#97F27F')
        self.light_green = wx.ListItemAttr()
        self.light_green.SetBackgroundColour('#E5FFE5')
        self.red = wx.ListItemAttr()
        self.red.SetBackgroundColour('#FF1616')
        self.yellow = wx.ListItemAttr()
        self.yellow.SetBackgroundColour('#FDEB22')

        self.Bind(wx.EVT_LIST_ITEM_RIGHT_CLICK, self.onRightClick)


    # right click menu
    def _init_menu(self, menu):
        menu.edit = wx.MenuItem(menu, wxID_MENUEDIT, _(u"Manually Edit"))
        menu.edit.SetBitmap(wx.Bitmap(utils.iconPath(u'edit.png'),
            wx.BITMAP_TYPE_PNG))

        menu.remove = wx.MenuItem(menu, wxID_MENUREMOVE, _(u"Remove Item"))
        menu.remove.SetBitmap(wx.Bitmap(utils.iconPath(u'errors.png'),
            wx.BITMAP_TYPE_PNG))

        menu.AppendItem(menu.edit)
        menu.AppendItem(menu.remove)
        #menu.AppendItem(parent.disable)

        self.Bind(wx.EVT_MENU, self.removeItem,
              id=wxID_MENUREMOVE)
        self.Bind(wx.EVT_MENU, self.manualEdit,
              id=wxID_MENUEDIT)

    # Popup the menu
    def onRightClick(self, event):
        if hasattr(self.parent, 'toRename') and len(self.parent.toRename) != 0:
            self.rightClickMenu = wx.Menu()
            self._init_menu(self.rightClickMenu)
            self.PopupMenu(self.rightClickMenu)

    # remove an item by deselecting from picker and re-previewing
    def removeItem(self, event):
        itemNames = []
        item = -1
        while True:
            item = self.GetNextItem(item, wx.LIST_NEXT_ALL, wx.LIST_STATE_SELECTED)
            itemName = self.parent.toRename[item][0][0]
            if item == -1:
                break
            else:
                itemNames.append(itemName)
        self.parent.picker.removeItemsByName(itemNames)
        self.parent.showPreview(event)

    # allow editing single item
    def manualEdit(self, event):
        new = False
        item = self.GetNextItem(-1, wx.LIST_NEXT_ALL, wx.LIST_STATE_SELECTED)

        original = self.parent.toRename[item][1]
        txt = original[0]

        dlg = editDialog.create(self,txt)

        if dlg.ShowModal() == wx.ID_OK:
            new = dlg.GetValue()
        dlg.Destroy()

        if new:
            self.parent.toRename[item][1][0] = new

    # get the info for an item from the list
    def getNames(self, item):
        item = self.parent.toRename[int(item)]
        original = item[0]
        renamed = item[1]
        return original, renamed

    def OnGetItemText(self, item, col):
        original, renamed = self.getNames(item)
        split = os.path.split(original[0])
        if self.showDirs:
            to_rename = renamed[0].lstrip(os.path.sep)
        else:
            to_rename = os.path.split(renamed[0])[1]
        if col == 0:
            return split[0]
        elif col == 1:
            return split[1]
        else:
            return to_rename

    def OnGetItemImage(self, item):
        pickerList = self.parent.picker.Panel.ItemList
        original, renamed = self.getNames(item)
        if original[1]:
            # show preview if available
            if self.parent.prefs['showPreviewIcons'] and\
              pickerList.thumbnails.has_key(original[0]):
                img = pickerList.thumbnails[original[0]]
            else:
                img = 1
        else:
            img = 0
        return img

    def GetPreviewAttr(self, item):
        original, renamed = self.getNames(item)
        # make bad/warn names stand out
        if item in self.parent.bad:
            return self.red
        elif item in self.parent.warn:
            return self.yellow
        # make items that will be renamed stand out
        elif self.parent.prefs['showPreviewHighlight'] and\
          (original[0] != renamed[0]):
            return self.light_green
        else:
            return None

    def GetRenameAttr(self, item):
        original, renamed = self.getNames(item)
        if renamed[1] == False:
            return None
        elif renamed[1]:
            return self.green
        elif renamed[1] == None:
            return self.red

    def OnGetItemAttr(self, item):
        if self.mode == 'preview':
            return self.GetPreviewAttr(item)
        elif self.mode == 'rename':
            return self.GetRenameAttr(item)



# ------------ the bottom panel of the main splitter
class bottomWindow(wx.Panel):
    def sizer(self):
        buttonSizer = self.buttonSizer = wx.BoxSizer(wx.HORIZONTAL)
        mainSizer = self.mainSizer = wx.BoxSizer(wx.VERTICAL)
        buttonSizer.Add(self.preview,0,wx.LEFT, 10)
        buttonSizer.Add(self.go,0,wx.LEFT, 10)
        buttonSizer.Add((-1,25),1)
        buttonSizer.Add(self.imgPreview,0,wx.ALIGN_CENTER|wx.RIGHT,5)
        buttonSizer.Add(self.thumbSize,0,wx.ALIGN_CENTER|wx.RIGHT,15)
        buttonSizer.Add(self.autoPreview,0,wx.ALIGN_CENTER|wx.RIGHT,5)
        mainSizer.Add(buttonSizer,0,wx.EXPAND|wx.ALIGN_LEFT|wx.TOP,5)
        mainSizer.Add(self.display,1,wx.EXPAND|wx.ALL, 5)
        self.SetSizerAndFit(mainSizer)


    def _init_ctrls(self, prnt):
        wx.Panel.__init__(self, id=wxID_BOTTOM_WINDOW, name=u'Panel',
                          parent=prnt, style=wx.TAB_TRAVERSAL)


        self.display = ListCtrl(self, -1, size=wx.Size(520, -1))

        #self.display.Bind(wx.EVT_LIST_ITEM_FOCUSED, self.main.onItemSelected,
        #                  self.display)

        self.display.Bind(wx.EVT_LIST_ITEM_SELECTED, self.main.onItemSelected,
                          self.display)

        self.preview = buttons.GenBitmapTextButton(self, wxID_MAIN_WINDOWPREVIEW,
                       wx.Bitmap(utils.iconPath(u'preview.png'),
                       wx.BITMAP_TYPE_PNG),_(u"Preview"), size=(-1, 26))
        self.Bind(wx.EVT_BUTTON, self.main.onPreviewButton,
                  id=wxID_MAIN_WINDOWPREVIEW)

        self.go = buttons.GenBitmapTextButton(self, wxID_MAIN_WINDOWGO,
                  wx.Bitmap(utils.iconPath(u'go.png'),wx.BITMAP_TYPE_PNG),
                  _(u"Go!"), size=(-1, 26))
        self.go.Enable(False)
        self.go.Bind(wx.EVT_BUTTON, self.main.renameItems, id=wxID_MAIN_WINDOWGO)

        self.imgPreview = wx.CheckBox(id=wxID_MAIN_WINDOWIMGPREVIEW,
          label=_(u"Show image thumbnails"), name=u'imgPreview', parent=self,
          style=0)
        self.imgPreview.SetToolTipString(_(u"Can slow preview considerably"))
        self.imgPreview.SetValue(False)
        self.imgPreview.Bind(wx.EVT_CHECKBOX, self.refreshPicker)

        self.thumbSize = wx.Choice(id=wxID_MAIN_WINDOWTHUMBSIZE,
                         choices=[u'32',u'64',u'128',u'256'],
                         name=u'dirsPlace', parent=self, size=wx.Size(62,-1),
                         style=0)
        self.thumbSize.SetSelection(1)
        self.thumbSize.SetToolTipString(_(u"Thumbnail Size"))
        self.thumbSize.Bind(wx.EVT_CHOICE, self.setThumbnailSize)

        if not utils.pilIsLoaded():
            self.imgPreview.SetValue(False)
            self.imgPreview.Enable(False)
            self.thumbSize.Enable(False)


        self.autoPreview = wx.CheckBox(id=wxID_MAIN_WINDOWAUTOPREVIEW,
          label=_(u"Automatic Preview"), name=u'autoPreview', parent=self,
          style=0)
        self.autoPreview.SetToolTipString(_(u"Disable when working with many items"))
        self.autoPreview.SetValue(True)


    def __init__(self, prnt, MainWindow):
        self.main = MainWindow
        self._init_ctrls(prnt)
        self.sizer()
        self.createUndoRedo(_("Undo"))
        # disable if no undo files exist (to lessen user confusion)
        if not os.path.exists(utils.stdPath(u'undo/original.bak')) or not\
          os.path.exists(utils.stdPath(u'undo/renamed.bak')):
            self.undoRedo.Enable(False)

    def refreshPicker(self, event):
        self.main.picker.refresh(event)

    # only re-preview if image preview is enabled and size has changed
    def setThumbnailSize(self, event):
        if hasattr(self.main.picker.Panel.ItemList, 'thumbSize'):
            thumbSize = self.main.picker.Panel.ItemList.thumbSize[0]
        else:
            thumbSize = int(event.GetString())
        if self.imgPreview.GetValue() and int(event.GetString()) != thumbSize:
            self.refreshPicker(event)

    # deletes and creates a new button, hack to change image
    def createUndoRedo(self,label):
        if label == _("Undo"):
            img = wx.Bitmap(utils.iconPath(u'undo.ico'),wx.BITMAP_TYPE_ICO)
        else:
            img = wx.Bitmap(utils.iconPath(u'redo.ico'),wx.BITMAP_TYPE_ICO)

        if hasattr(self, 'undoRedo'):
            self.undoRedo.Destroy()

        self.undoRedo = buttons.GenBitmapTextButton(self, wxID_MAIN_WINDOWUNDOREDO,img, label, size=(-1, 26))
        # do not use wx.EVT_BUTTON, causes seg fault
        self.undoRedo.Bind(wx.EVT_LEFT_DOWN, self.main.undoRename, id=wxID_MAIN_WINDOWUNDOREDO)
        self.buttonSizer.Insert(2,self.undoRedo,proportion=0,flag=wx.LEFT,border=25)
        self.Layout()

    def setUndoRedoType(self,op):
        if op == 'check':
            if self.undoRedo.GetLabel() == _("Undo"):
                self.createUndoRedo(_("Redo"))
            else:
                self.createUndoRedo(_("Undo"))
        elif op == 'undo':
            self.createUndoRedo(_("Undo"))



# ------------ the main splitter
class MySplitter(wx.SplitterWindow):
    def __init__(self, parent, ID):
        wx.SplitterWindow.__init__(self, parent, ID,
                        style=wx.SP_LIVE_UPDATE|wx.SP_3DSASH|wx.SP_NO_XP_THEME)


# ------------ the main application class:
class MainWindow(wx.Frame):
    def _init_coll_menuBar_Menus(self, parent):
        # menu bar
        parent.Append(menu=self.menuFile, title=_(u"File"))
        parent.Append(menu=self.menuPicker, title=_(u"Picker"))
        parent.Append(menu=self.menuEdit, title=_(u"Edit"))
        parent.Append(menu=self.menuHelp, title=_(u"Help"))
        self.menuPicker.getAllMenu.Enable(False)
        self.menuPicker.getNoneMenu.Enable(False)
        self.menuFile.GoMenu.Enable(False)

    def _init_coll_menuEdit_Items(self, parent):
        # EDIT MENU
        parent.PrefsMenu = wx.MenuItem(parent,
         wxID_MAIN_WINDOWMENUEDITPREFERENCES, _(u"Preferences"),
         self.makeSpace(_(u"Change your preferences")))

        parent.langMenu = wx.MenuItem(parent,
         wxID_MAIN_WINDOWMENUEDITLANG, _(u"Language"),
         self.makeSpace(_(u"Change the language")))

        parent.PrefsMenu.SetBitmap(wx.Bitmap(\
            utils.iconPath(u'preferences.ico'), wx.BITMAP_TYPE_ICO))
        parent.AppendItem(parent.PrefsMenu)

        parent.langMenu.SetBitmap(wx.Bitmap(\
            utils.iconPath(u'language.png'),wx.BITMAP_TYPE_PNG))
        parent.AppendItem(parent.langMenu)

        self.Bind(wx.EVT_MENU, self.onMenuEditPreferencesMenu,
              id=wxID_MAIN_WINDOWMENUEDITPREFERENCES)
        self.Bind(wx.EVT_MENU, self.languageSelect,
              id=wxID_MAIN_WINDOWMENUEDITLANG)


    def _init_coll_menuHelp_Items(self, parent):
        # HELP MENU
        parent.aboutMenu = wx.MenuItem(parent,
         wxID_MAIN_WINDOWMENUHELPABOUT, _(u"About"),
         self.makeSpace(_(u"Display general information about Metamorphose")))
        parent.aboutMenu.SetBitmap(wx.Bitmap(utils.iconPath(u'about.ico'),
            wx.BITMAP_TYPE_ICO))

        parent.helpMenu = wx.MenuItem(parent,
         wxID_MAIN_WINDOWMENUHELPHELP, _(u"&Help\tF1"),
         self.makeSpace(_(u"How to use Metamorphose")))
        parent.helpMenu.SetBitmap(wx.Bitmap(utils.iconPath(u'help.ico'),
            wx.BITMAP_TYPE_ICO))

        parent.examplesMenu = wx.MenuItem(parent,
         wxID_MAIN_WINDOWMENUHELPEXAMPLES, _(u"&Examples\tF2"),
         self.makeSpace(_(u"Some useful examples")))
        parent.examplesMenu.SetBitmap(wx.Bitmap(\
            utils.iconPath(u'examples.ico'),wx.BITMAP_TYPE_ICO))

        parent.FormatHelpMenu = wx.MenuItem(parent,
         wxID_MAIN_WINDOWMENUHELPFORMATHELP, _(u"&Date/Time Formats"),
         self.makeSpace(_(u"Display a reference for Date & Time formats")))
        parent.FormatHelpMenu.SetBitmap(wx.Bitmap(\
            utils.iconPath(u'date_time.ico'),wx.BITMAP_TYPE_ICO))

        parent.REhelpMenu = wx.MenuItem(parent,
         wxID_MAIN_WINDOWMENUHELPREHELP, _(u"&Regular Expressions"),
         self.makeSpace(_(u"Display a regular expression reference")))
        parent.REhelpMenu.SetBitmap(wx.Bitmap(utils.iconPath(u're.ico'),
            wx.BITMAP_TYPE_ICO))

        parent.AppendItem(parent.aboutMenu)
        parent.AppendItem(parent.helpMenu)
        parent.AppendItem(parent.examplesMenu)
        parent.AppendItem(parent.FormatHelpMenu)
        parent.AppendItem(parent.REhelpMenu)

        self.Bind(wx.EVT_MENU, self.onMenuHelpAboutMenu,
              id=wxID_MAIN_WINDOWMENUHELPABOUT)
        self.Bind(wx.EVT_MENU, self.onMenuHelpHelpMenu,
              id=wxID_MAIN_WINDOWMENUHELPHELP)
        self.Bind(wx.EVT_MENU, self.showSmallHelp,
              id=wxID_MAIN_WINDOWMENUHELPEXAMPLES)
        self.Bind(wx.EVT_MENU, self.showSmallHelp,
              id=wxID_MAIN_WINDOWMENUHELPFORMATHELP)
        self.Bind(wx.EVT_MENU, self.showSmallHelp,
              id=wxID_MAIN_WINDOWMENUHELPREHELP)


    def _init_coll_menuPicker_Items(self, parent):
        # PICKER MENU
        parent.browseMenu = wx.MenuItem(parent, wxID_MAIN_WINDOWMENUPICKERBROWSE,
            _(u"&Browse...\tF4"),
            self.makeSpace(_(u"Browse for path")))
        parent.browseMenu.SetBitmap(wx.Bitmap(\
            utils.iconPath(u'browse.png'), wx.BITMAP_TYPE_PNG))
        parent.okMenu = wx.MenuItem(parent, wxID_MAIN_WINDOWMENUPICKEROK,
            _(u"&Refresh\tF5"),
            self.makeSpace(_(u"Load or reload current path")))
        parent.okMenu.SetBitmap(wx.Bitmap(utils.iconPath(u'reload.png'),
            wx.BITMAP_TYPE_PNG))

        parent.getAllMenu = wx.MenuItem(parent, wxID_MAIN_WINDOWMENUPICKERALL,
            _(u"Select &All\tctrl+A"),
            self.makeSpace(_(u"Select all items in picker")))
        parent.getAllMenu.SetBitmap(wx.Bitmap(\
            utils.iconPath(u'selectAll.png'), wx.BITMAP_TYPE_PNG))
        parent.getNoneMenu = wx.MenuItem(parent, wxID_MAIN_WINDOWMENUPICKERNONE,
            _(u"Select &None\tctrl+N"),
            self.makeSpace(_(u"Deselect all items in picker")))
        parent.getNoneMenu.SetBitmap(wx.Bitmap(\
            utils.iconPath(u'selectNone.png'), wx.BITMAP_TYPE_PNG))
        parent.walkMenu = wx.MenuItem(parent, wxID_MAIN_WINDOWMENUPICKERWALK,
            _(u"Recursive &selection\tctrl+R"),
            self.makeSpace(_(u"Get all files in directory and sub-directories, but no folders")))
        parent.walkMenu.SetBitmap(wx.Bitmap(utils.iconPath(u'walk.png'),
            wx.BITMAP_TYPE_PNG))

        parent.AppendItem(parent.browseMenu)
        parent.AppendItem(parent.okMenu)
        parent.AppendSeparator()
        parent.AppendItem(parent.getAllMenu)
        parent.AppendItem(parent.getNoneMenu)
        parent.AppendItem(parent.walkMenu)

        self.Bind(wx.EVT_MENU, self.picker.browseForPath,
              id=wxID_MAIN_WINDOWMENUPICKERBROWSE)
        self.Bind(wx.EVT_MENU, self.picker.setPanelPath,
              id=wxID_MAIN_WINDOWMENUPICKEROK)
        self.Bind(wx.EVT_MENU, self.picker.selectNoItemsInPanel,
              id=wxID_MAIN_WINDOWMENUPICKERNONE)
        self.Bind(wx.EVT_MENU, self.picker.selectAllItemsInPanel,
              id=wxID_MAIN_WINDOWMENUPICKERALL)
        self.Bind(wx.EVT_MENU, self.picker.walkFromMenu,
              id=wxID_MAIN_WINDOWMENUPICKERWALK)


    def _init_coll_menuFile_Items(self, parent):
        # FILE MENU
        parent.LoadIniMenu = wx.MenuItem(parent, wxID_MAIN_WINDOWMENUFILELOADINI,
            _(u"&Load Config\tctrl+L"),
            self.makeSpace(_(u"Load configuration from file")))
        parent.LoadIniMenu.SetBitmap(wx.Bitmap(utils.iconPath(u'loadIni.png'),
            wx.BITMAP_TYPE_PNG))
        parent.SaveIniMenu = wx.MenuItem(parent, wxID_MAIN_WINDOWMENUFILESAVEINI,
            _(u"&Save Config\tctrl+S"),
            self.makeSpace(_(u"Save configuration to file")))
        parent.SaveIniMenu.SetBitmap(wx.Bitmap(utils.iconPath(u'saveIni.png'),
            wx.BITMAP_TYPE_PNG))

        parent.SaveLog = wx.MenuItem(parent, wxID_MAIN_WINDOWMENUFILESAVELOG,
            _(u"&Export to log file"),
            self.makeSpace(_(u"Save current snapshot to a log file")))
        parent.SaveLog.SetBitmap(wx.Bitmap(utils.iconPath(u'CSVto.png'),
            wx.BITMAP_TYPE_PNG))
        parent.Import = wx.MenuItem(parent,
         wxID_MAIN_WINDOWMENUFILEIMPORT, _(u"&Import from log file"),
         self.makeSpace(_(u"Load a snapshot")))
        parent.Import.SetBitmap(wx.Bitmap(utils.iconPath(u'CSVfrom.png'),
            wx.BITMAP_TYPE_PNG))

        parent.PreviewMenu = wx.MenuItem(parent, wxID_MAIN_WINDOWMENUFILEPREVIEW,
            _(u"&Preview\tF7"), self.makeSpace(_(u"Preview selection")))
        parent.PreviewMenu.SetBitmap(wx.Bitmap(utils.iconPath(u'preview.png'),
            wx.BITMAP_TYPE_PNG))
        parent.GoMenu = wx.MenuItem(parent, wxID_MAIN_WINDOWMENUFILEGO,
            _(u"&Go !\tF8"), self.makeSpace(_(u"Rename selection")))
        parent.GoMenu.SetBitmap(wx.Bitmap(utils.iconPath(u'go.png'),
            wx.BITMAP_TYPE_PNG))

        parent.exitMenu = wx.MenuItem(parent, wxID_MAIN_WINDOWMENUFILEEXIT,
            _(u"&Quit\tctrl+Q"), self.makeSpace(_(u"Quit Metamorphose")))
        parent.exitMenu.SetBitmap(wx.Bitmap(utils.iconPath(u'exit.png'),
            wx.BITMAP_TYPE_PNG))

        parent.AppendItem(parent.LoadIniMenu)
        parent.AppendItem(parent.SaveIniMenu)
        parent.AppendSeparator()
        parent.AppendItem(parent.SaveLog)
        parent.SaveLog.Enable(False)
        parent.AppendItem(parent.Import)
        parent.AppendSeparator()
        parent.AppendItem(parent.PreviewMenu)
        parent.AppendItem(parent.GoMenu)
        parent.AppendSeparator()
        parent.AppendItem(parent.exitMenu)

        self.Bind(wx.EVT_MENU, self.saveAsText,
              id=wxID_MAIN_WINDOWMENUFILESAVELOG)
        self.Bind(wx.EVT_MENU, self.importSelection,
              id=wxID_MAIN_WINDOWMENUFILEIMPORT)
        self.Bind(wx.EVT_MENU, self.saveCfg,
              id=wxID_MAIN_WINDOWMENUFILESAVEINI)
        self.Bind(wx.EVT_MENU, self.loadCfg,
              id=wxID_MAIN_WINDOWMENUFILELOADINI)
        self.Bind(wx.EVT_MENU, self.onPreviewButton,
              id=wxID_MAIN_WINDOWMENUFILEPREVIEW)
        self.Bind(wx.EVT_MENU, self.renameItems,
              id=wxID_MAIN_WINDOWMENUFILEGO)
        self.Bind(wx.EVT_MENU, self.onMenuFileExitMenu,
              id=wxID_MAIN_WINDOWMENUFILEEXIT)

    def note_book(self):
        parent = self.notebook

        # init Core classes
        self.picker = picker.Core(parent, self)
        self.sorting = sorting.Core(parent, self)
        self.errors = errors.Core(parent, self)
        self.renamer = renamer.Core(parent, self)

        # add notebook pages to notebook
        parent.AddPage(self.picker.Panel, _(u"Picker"))
        parent.AddPage(self.renamer.Panel, _(u"- Renamer -"))
        parent.AddPage(self.sorting.Panel, _(u"Sorting"))
        parent.AddPage(self.errors.Panel, _(u"Errors/Warnings: %s")%0)

        # list containing notebook images
        il = wx.ImageList(16, 16)
        img0 = il.Add(wx.Bitmap(utils.iconPath(u'picker.ico'),
                      wx.BITMAP_TYPE_ICO))
        img1 = il.Add(wx.Bitmap(utils.iconPath(u'main.ico'),
                      wx.BITMAP_TYPE_ICO))
        img2 = il.Add(wx.Bitmap(utils.iconPath(u'sorting.ico'),
                      wx.BITMAP_TYPE_ICO))
        img3 = il.Add(wx.Bitmap(utils.iconPath(u'errors.ico'),
                      wx.BITMAP_TYPE_ICO))

        # set images to pages
        parent.AssignImageList(il)
        parent.SetPageImage(0, img0)
        parent.SetPageImage(1, img1)
        parent.SetPageImage(2, img2)
        parent.SetPageImage(3, img3)
        # required to make XP theme colours ok
        self.notebook.SetBackgroundColour(self.notebook.GetThemeBackgroundColour())

    def sizer(self):
        mainSizer = self.mainSizer = wx.BoxSizer(wx.VERTICAL)
        mainSizer.Add(self.splitter,1,wx.EXPAND|wx.ALL, 5)
        self.SetSizerAndFit(mainSizer)

    def _init_utils(self):
        self.menuFile = wx.Menu()
        self.menuPicker = wx.Menu()
        self.menuEdit = wx.Menu()
        self.menuHelp = wx.Menu()
        self.menuBar = wx.MenuBar()
        self._init_coll_menuFile_Items(self.menuFile)
        self._init_coll_menuPicker_Items(self.menuPicker)
        self._init_coll_menuEdit_Items(self.menuEdit)
        self._init_coll_menuHelp_Items(self.menuHelp)
        self._init_coll_menuBar_Menus(self.menuBar)
        self.SetMenuBar(self.menuBar)

    def _init_ctrls(self, prnt):
        self.SetTitle(u"Métamorphose 2 (beta)")
        self.SetBackgroundColour(wx.NullColour)
        self.SetAutoLayout(False)
        self.SetStatusBarPane(0)
        self.SetIcon(wx.Icon(utils.iconPath(u'metamorphose.ico'), wx.BITMAP_TYPE_ICO))
        self.SetThemeEnabled(True)
        self.SetFont(wx.Font(9, wx.DEFAULT, wx.NORMAL, wx.NORMAL))

        self.statusBar1 = ESB.EnhancedStatusBar(id=wxID_MAIN_WINDOWSTATUSBAR1,
              name=u'statusBar1', parent=self)
        self.SetStatusBar(self.statusBar1)

        self.statusImage = wx.StaticBitmap(bitmap=self.statusImages[u'eyes'],
              id=wxID_MAIN_WINDOWSTATUSIMAGE, name=u'statusImage',
              parent=self.statusBar1, size=wx.Size(-1, 16), style=0)
        self.statusBar1.AddWidget(self.statusImage, ESB.ESB_ALIGN_LEFT)

        self.splitter = MySplitter(self, wxID_SELECTIONAREA)

        # notebook
        self.notebook = wx.Notebook(id=wxID_MAIN_WINDOWNOTEBOOK,
              name=u'notebook', parent=self.splitter,
              style=wx.NB_TOP|wx.NO_BORDER)
        self.notebook.SetThemeEnabled(True)

        self.bottomWindow = bottomWindow(self.splitter, self)
        self.SetMinSize(wx.Size(-1,600))

        self.splitter.SetMinimumPaneSize(40)

        if wx.Platform == '__WXGTK__':
            split = -210
        elif wx.Platform == '__WXMSW__':
            split = -200
        else:
            split = -205
        self.splitter.SplitHorizontally(self.notebook, self.bottomWindow, split)


    def usage(self):
        print
        print "Metamorphose 2.%s\nRunning on Python %s, wxPython %s"%(self.version, platform.python_version(), utils.getWxVersion())
        print "Copyright (C) 2006-2009 ianare sevi"
        print "<http://file-folder-ren.sourceforge.net/>\n"
        print "Metamorphose is a graphical mass renaming program for files and folders."
        print "However, there are some command line options :\n"
        print "-h,  --help       Show this screen and exit"
        print "-t,  --timer      Show time taken to complete operations"
        print "-d,  --debug      Show debugging information"
        print "-p=, --path=      Specify a directory to load"
        print "-c=, --config=    Specify a Metamorphose2 config file to load"
        print "-a=, --auto=      Specify automatic mode level (when also using -c option) :"
        print "                    0 = do not preview items (will overide preferences)"
        print "                    1 = auto preview items (will overide preferences)"
        print "                    2 = auto rename items"
        print "                    3 = auto rename items and exit on success"
        print "-l=, --language=  Overide prefered language :"
        print "                    en_US"
        print "                    fr"
        print
        print "If no other options are given, you may specify a path to open :"
        print "$ metamorphose2 /srv/samba/Windows Likes Spaces"
        print


    def __init__(self, prnt):

        # Important variables needed throughout the application classes
        #
        self.version = utils.getVersion()
        self.warn = [] # warnings
        self.bad = [] # errors
        self.errorLog = [] # all errors go here
        self.items = [] # items to rename
        self.spacer = u" "*6 # spacer for status messages (to clear image)
        self.REmsg = False # regular expression error
        self.recursiveFolderOn = False # if true, will need to sort items before renaming
        self.language = '' # to be set later
        self.prefs = {} # preferences, to be set later
        self.autoModeLevel = False # automatic mode level
        self.showTimes = False # show processing times
        self.debug = False # show debug info

        ### handle command line arguments
        shortOptions = "htdp:vc:va:vl:v"
        longOptions = ["help", "timer", "debug", "path=", "config=", "auto=", "language="]
        try:
            opts, args = getopt.getopt(sys.argv[1:], shortOptions, longOptions)
        except getopt.GetoptError, err:
            # print help information and exit:
            print str(err)
            sys.exit(2)

        # set options
        path = False
        configFilePath = False

        # standardize on using '='
        def stripLeading(a):
            if '=' in a:
                a = a.lstrip('=')
            return a

        def exit(message):
            print "Command line error :"
            print message
            print
            sys.exit()

        if sys.argv[1:] != [] and opts == []:
            path = ''
            for chunk in sys.argv[1:]:
                path += chunk + ' '

        for o, a in opts:
            # show processing times
            if o in ("-t", "--timer"):
                self.showTimes = True
                print "Showing processing times"
            # show debug messages
            elif o in ("-d", "--debug"):
                self.debug = True
                print "Running in debug mode"
                print "Version : " + self.version
                print "Python %s, wxPython %s"%(platform.python_version(), utils.getWxVersion())
            # set path
            elif o in ("-p", "--path"):
                path = stripLeading(a)
            # show help and exit
            elif o in ("-h", "--help"):
                self.usage()
                sys.exit()
            # use a config path
            elif o in ("-c", "--config"):
                configFilePath = stripLeading(a)
           # set auto level
            elif o in ("-a", "--auto"):
                level = stripLeading(a)
                if level not in '0123':
                    exit("Invalid auto mode level : '%s'"%level)
                elif not configFilePath:
                    exit("Auto mode level must be used with configuration file ( '-c' or '--config' option )")
                else:
                    print "Auto mode level set : %s"%level
                    self.autoModeLevel = int(level)
           # specify the language
            elif o in ("-l", "--language"):
                self.language = stripLeading(a)

        #
        wx.Frame.__init__(self, id=wxID_MAIN_WINDOW, name=u'MainWindow',
              parent=prnt, pos=wx.Point(-1, -1), size=wx.Size(-1, -1),
              style=wx.DEFAULT_FRAME_STYLE)

        # first run?
        utils.makeInitialEnvironment()

        # get language from file if not specified from command line
        if self.language == '':
            try:# see if language file exist
                langIni = codecs.open(utils.stdPath(u'language.ini'),'r', 'utf-8')
            except IOError:# have user choose language
                language = self.languageSelect(0)
            else:# get language from file
                language = langIni.read().strip()
        else:
            language = self.language

        # reference:
        locales = {
            #u'ar' : (wx.LANGUAGE_ARABIC, u'ar_SA.UTF-8'),
            #u'de' : (wx.LANGUAGE_GERMAN, u'de_DE.UTF-8'),
            #u'el' : (wx.LANGUAGE_GREEK, u'el_GR.UTF-8'),
            #u'en_UK' : (wx.LANGUAGE_ENGLISH_UK, u'en_GB.UTF-8'),
            u'en_US' : (wx.LANGUAGE_ENGLISH_US, u'en_US.UTF-8'),
            #u'es' : (wx.LANGUAGE_SPANISH, u'es_ES.UTF-8'),
            u'fr' : (wx.LANGUAGE_FRENCH, u'fr_FR.UTF-8'),
            #u'he' : (wx.LANGUAGE_HEBREW, u'he_IL.UTF-8'),
            #u'hu' : (wx.LANGUAGE_HUNGARIAN, u'hu_HU.UTF-8'),
            #u'it' : (wx.LANGUAGE_ITALIAN, u'it_IT.UTF-8'),
            #u'ja' : (wx.LANGUAGE_JAPANESE, u'ja_JP.UTF-8'),
            #u'pl' : (wx.LANGUAGE_POLISH, u'pl_PL.UTF-8'),
            #u'pt_BR' : (wx.LANGUAGE_PORTUGUESE_BRAZILIAN, u'pt_BR.UTF-8'),
            #u'ru' : (wx.LANGUAGE_RUSSIAN, u'ru_RU.UTF-8'),
            #u'sv' : (wx.LANGUAGE_SWEDISH, u'sv_SE.UTF-8'),
            #u'tr' : (wx.LANGUAGE_TURKISH, u'tr_TR.UTF-8'),
            #u'zh_CN' : (wx.LANGUAGE_CHINESE_SIMPLIFIED, u'zh_CN.UTF-8'),
        }

        # right-to-left languages:
        right_left_languages = ('ar','dv','fa','ha','he','ps','ur','yi')

        try:
            locales[language]
        except KeyError:
            msg = u"Could not initialise language: '%s'.\nContinuing in "%language
            msg += u"American English (en_US)\n"
            print msg
            language = 'en_US'

        # needed for calendar and other things, send all logs to stderr
        wx.Log.SetActiveTarget(wx.LogStderr())

        # set locale and language
        mylocale = wx.Locale(locales[language][0], wx.LOCALE_LOAD_DEFAULT)
        Lang = gettext.translation(u'metamorphose2', utils.localePath(language),
                                   languages=[locales[language][1]])
        Lang.install(unicode=True)

        # set some globals
        if language not in right_left_languages:
            self.langLTR = True
            self.alignment = wx.ALIGN_LEFT
        else:
            self.langLTR = False
            self.alignment = wx.ALIGN_RIGHT
        self.language = language
        self.encoding = unicode(locale.getlocale()[1])

        # to get some language settings to display properly:
        unixPlatforms = ('Linux', 'FreeBSD')
        if platform.system() in unixPlatforms:
            try:
                os.environ['LANG'] = locales[language][1]
            except (ValueError, KeyError):
                pass


        # import these modules here since they need language settings activated
        global renamer
        import renamer
        global configs
        import configs
        global picker
        import picker

        # initialise preferences
        self.prefs = preferences.Methods(MainWindow=self).getPrefs()

        # icons used for status bar messages
        self.statusImages = {
            u'failed' : wx.Bitmap(utils.iconPath(u'failed_sb.ico'),
                        wx.BITMAP_TYPE_ICO),
            u'wait' : wx.Bitmap(utils.iconPath(u'wait.png'),
                        wx.BITMAP_TYPE_PNG),
            u'warn' : wx.Bitmap(utils.iconPath(u'warn_sb.ico'),
                        wx.BITMAP_TYPE_ICO),
            u'complete' : wx.Bitmap(utils.iconPath(u'complete.ico'),
                        wx.BITMAP_TYPE_ICO),
            u'eyes' : wx.Bitmap(utils.iconPath(u'eyes.png'),
                        wx.BITMAP_TYPE_PNG),
            }

        # build main GUI
        self._init_ctrls(prnt)

        # clear undo (if set in preferences):
        if self.prefs[u'clearUndo']:
            try:
                originalFile = codecs.open(utils.stdPath(u'undo/original.bak'),
                                           'w', "utf-8")
                originalFile.write('')
                renamedFile = codecs.open(utils.stdPath(u'undo/renamed.bak'),
                                          'w', "utf-8")
                renamedFile.write('')
            except IOError, error:
                utils.makeErrMsg(_(u"%s\n\nCould not clear undo")%error,
                                 _(u"Error"))
                pass

        # construct rest of GUI:
        self.note_book()
        self._init_utils()
        self.sizer()
        # call this after sizer to place properly:
        self.Center(wx.HORIZONTAL|wx.VERTICAL)

        # Load config from command line
        if configFilePath:
            configs.loadConfig(self, configFilePath)

        # Set root directory from command line arguments:
        if path:
            path = path.rstrip()
            self.picker.Panel.path.SetValue(path)
            self.picker.setPanelPath(True)


##### MISC STUFF: ##############################################################
    # Little functions to make repetative stuff easier to implement.

    # add spacing for translation
    def makeSpace(self,statusText):
        #if self.langLTR:
        #    return self.spacer+statusText
        #else:
        return self.spacer+statusText

    # this function is called by basically every widget in the app.
    # It calls the preview function, if the user has set so in preferences:
    def showPreview(self, event):
        if event and self.bottomWindow.autoPreview.GetValue():
            self.onPreviewButton(False)


    # open selection dialog for language:
    def languageSelect(self, event):
        # if triggered from event, language has already been loaded,
        # and we will be changing it. No event = no language file
        if event == 0:
            Title = u'\m/ (>_<) \m/'
        else:
            Title = _(u"Language")
            event = self.language

        # show the language choices
        dlg = langSelect.create(self, Title, event)
        if dlg.ShowModal() == wx.ID_OK:
            language = dlg.GetLanguage()
            dlg.Destroy()
            if self.language != language:
                self.changeLanguage(language, event)
            return language

    # write language.ini file
    def changeLanguage(self, language, event):
        try:
            langFile = codecs.open(utils.stdPath(u'language.ini'),'w', 'utf-8')
        except IOError, error:
            utils.makeErrMsg(unicode(error), u"Error")
            pass
        else:
            langFile.write(language)
            langFile.close()
        if event:
            msg = _(u"\n\nYou will need to restart Metamorphose to change the language.")
            msg += _(u"\n\nClose out of Metamorphose now?")
            msg = language + msg
            dlg = wx.MessageDialog(self, msg,
              _(u"Change the Language"), wx.YES_NO | wx.YES_DEFAULT)
            # Restart app automatically if not in windows
            if dlg.ShowModal() == wx.ID_YES:
                if platform.system() != 'Windows':
                    if wx.Process.Open(utils.realPath(sys.argv[0])):
                        self.Close()
                else:
                    self.Close()

    # Set status bar text and image:
    def setStatusMsg(self, msg, select):
        self.statusImage.SetBitmap(self.statusImages[select])
        self.SetStatusText(self.makeSpace(msg))

        if self.debug:
            print msg



##### MENU ACTIONS: ############################################################
    def onMenuFileExitMenu(self, event):
        self.Close()

    def saveCfg(self, event):
        configs.saveConfig(self)

    def loadCfg(self, event):
        configs.loadConfigDialog(self)

    #------- export current selection as CSV file -#
    def saveAsText(self, event):
        if hasattr(self,'toRename'):
            CSVfile = ""
            q = self.prefs['logEnclose']
            sep = self.prefs['logSeparator']
            ext = self.prefs['logFextension']

            if self.showTimes:
                t = time.time()

            # create file contents
            for original, renamed in self.toRename:
                CSVfile += unicode(q+original[0]+q+sep+q+renamed[0]+q+'\n')

            if self.showTimes:
                print "Export file contents for %s items : %s"%(len(self.toRename), (time.time() - t))

            # trigered by menu, allow to choose output file
            if event:
                dlg = wx.FileDialog(self, message=_(u"Save current items as ..."),
                      defaultDir='', defaultFile=u'',
                      wildcard=_(u"Log file (*.%s)")%ext + u'|*.%s'%ext,
                      style=wx.SAVE|wx.OVERWRITE_PROMPT
                      )
                if dlg.ShowModal() == wx.ID_OK:
                    # attempt to write file
                    utils.writeFile(dlg.GetPath(), CSVfile)
                dlg.Destroy()
            # auto logging, generate file name
            else:
                file = time.strftime("undo_%Y-%m-%d_%Hh%Mm%Ss",
                       time.localtime())+'.'+self.prefs['logFextension']
                path = os.path.join(self.prefs[u'logLocation'], file)
                utils.writeFile(path, CSVfile)



    #------- imports selection from txt file -#
    def importSelection(self,event):
        if self.prefs['logLocation'] != '':
            path = self.prefs['logLocation']
        else:
            path = utils.stdPath(u'undo')

        # settings from preferences
        ext = self.prefs['logFextension']
        q = self.prefs['logEnclose']
        sep = self.prefs['logSeparator']

        wildOnes = _(u"Log file")+u" (*.%s)|*.%s|"%(ext,ext)+\
                   _(u"All files")+"(*.*)|*.*"

        # file open dialog
        dlg = wx.FileDialog(
            self, message=_(u"Import selection from ..."), defaultDir=path,
            defaultFile=u'', wildcard=wildOnes,
            style=wx.OPEN|wx.CHANGE_DIR)

        if dlg.ShowModal() == wx.ID_OK:
            original = []
            renamed = []
            unloadbles = ""
            f = dlg.GetPath()

            # undo files have order reversed
            if os.path.basename(f)[0:5] == 'undo_':
                o = 1
                r = 0
            else:
                o = 0
                r = 1

            f = codecs.open(f, 'r', 'utf_8')
            i = 0
            for line in f.readlines():
                i += 1
                line = line.lstrip(q).rstrip(q+'\n').split(q+sep+q)
                # check for existance of every item
                if os.path.exists(line[o]):
                    original.append((line[o],1))
                    renamed.append([line[r],False])
                else:
                    unloadbles += "%s\n"%line[o]

            # set up for display
            self.toRename = zip(original,renamed)
            self.picker.Panel.path.SetValue('')
            self.picker.clearALL()

            # only allow rename if there are items
            if len(self.toRename) != i:
                msg = _(u"These items could not be loaded, it doesn't look like they exist :\n\n")
                msg += unloadbles
                dlg = wx.lib.dialogs.ScrolledMessageDialog(self, msg,
                      "Could not load all items")
                dlg.ShowModal()
            if len(self.toRename) != 0:
                self.bottomWindow.go.Enable(True)
                self.menuFile.GoMenu.Enable(True)
                self.currentItem = None
                self.displayResults(True)
            f.close()


    # opens help dialog:
    def onMenuHelpHelpMenu(self, event):
        helpDiag.create(self).Show()

    # opens generic html help dialog:
    def showSmallHelp(self, event):
        if event.GetId() == wxID_MAIN_WINDOWMENUHELPREHELP:
            helpFile=u'REhelp.html'
            Title=_(u"Regular Expression Help")
            Icon=u're'
        elif event.GetId() == wxID_MAIN_WINDOWMENUHELPFORMATHELP:
            helpFile=u'format.html'
            Title=_(u"Date and Time Formatting Help")
            Icon=u'date_time'
        elif event.GetId() == wxID_MAIN_WINDOWMENUHELPEXAMPLES:
            helpFile=u'examples.html'
            Title=_(u"Examples")
            Icon=u'examples'
        return utils.SmallHelp(self,self,helpFile,Title,Icon).Show()

    # opens about dialog:
    def onMenuHelpAboutMenu(self, event):
        aboutDlg = about.create(self)
        aboutDlg.CentreOnParent()
        aboutDlg.ShowModal()
        aboutDlg.Destroy()

    # opens preferences dialog:
    def onMenuEditPreferencesMenu(self, event):
        prefDiag = preferences.createDialog(self)
        prefDiag.ShowModal()
        prefDiag.Destroy()


##### UNDO: ####################################################################
    # Grabs names from .bak files and reverts them to their originals.
    # Keep this separate from csv file functions for safety/stability
    def undoRename(self, event):
        original = []
        renamed = []
        utils.setBusy(True)
        try:
            originalFile = codecs.open(utils.stdPath(u'undo/original.bak'),'r',
                           'utf-8')
            renamedFile = codecs.open(utils.stdPath(u'undo/renamed.bak'),'r',
                          'utf-8')
            for line in originalFile:
                original.append([line.strip(),False])
            for line in renamedFile:
                renamed.append([line.strip(),1])

        except IOError, error:
            utils.makeErrMsg(_(u"%s\n\nUndo Failed !!")%error, _(u"Error"))
            pass
        else:
            def getName(x): return x[0]
            commonPrefix = os.path.commonprefix(map(getName,original)).rstrip(os.sep)
            if os.path.exists(commonPrefix):
                self.picker.Panel.path.SetValue(commonPrefix)
                self.picker.Panel.dirPicker.SetPath(commonPrefix)

            if not len(original) == 0:
                self.toRename = zip(renamed,original)#reverse order from original rename!
                self.currentItem = None
                self.displayResults()
                self.renameItems(u'undo')
                self.menuFile.SaveLog.Enable(False)
            else:
                self.setStatusMsg(_(u"Nothing to undo"), u'eyes')

        self.bottomWindow.setUndoRedoType('check')
        utils.setBusy(False)



#### CHANGE ITEM POSITION IN LIST ##############################################
    # set the selected item
    def onItemSelected(self, event):
        self.currentItem = event.m_itemIndex

    # move the selected item
    def changeItemOrder(self, change):
        if self.currentItem != None:
            try:
                moveTo = self.currentItem + change
            except TypeError:
                if change == u'top':
                    moveTo = 0
                else:
                    moveTo = len(self.items)-1
                pass

            if moveTo > len(self.items):
                moveTo = len(self.items)-1
            if moveTo < 0:
                moveTo = 0

            old = self.items[self.currentItem]
            new = self.items[moveTo]

            del self.items[self.currentItem]
            self.items.insert(moveTo,old)

            self.currentItem = moveTo
            self.onPreviewButton(0)


##### NAME GENERATION ##########################################################
    def onPreviewButton(self, event):
        utils.setBusy(True)
        self.renamer.engine.generateNames(event)
        utils.setBusy(False)


#### DISPLAY RESULTS ###########################################################
    """
    Resets the bottom preview virtual list, then sets amount of items and the
    correct imagelist to use.
    """
    def displayResults(self, showDirs=False):
        self.errors.clearErrors()

        pickerList = self.picker.Panel.ItemList
        display = self.bottomWindow.display
        usedOperations = self.renamer.Panel.usedOperations
        # show complete path in new name
        if not showDirs:
            for i in range(usedOperations.GetItemCount()):
                op = usedOperations.GetItemText(i)
                if re.search(_("directory"),op):
                    showDirs = True
                    break
        display.showDirs = showDirs
        display.mode = 'preview'

        # set the image list
        if self.prefs['showPreviewIcons']:
            display.imgs = pickerList.imgs
        else:
            display.imgs = wx.ImageList(16,16)
            display.folderIco = display.imgs.Add(wx.Bitmap(\
              utils.iconPath(u'folder16.png'),wx.BITMAP_TYPE_PNG))
            display.fileIco = display.imgs.Add(wx.Bitmap(\
              utils.iconPath(u'file16.png'),wx.BITMAP_TYPE_PNG))
        display.SetImageList(display.imgs, wx.IMAGE_LIST_SMALL)

        display.DeleteAllItems()
        display.SetItemCount(len(self.toRename))

        # auto resize column:
        #display.SetColumnWidth(2,-1)
        if display.GetColumnWidth(2) < 135:
            display.SetColumnWidth(2,135)

        # show the last warning/error
        if self.warn != []:
            display.EnsureVisible(self.warn[-1])
        elif self.bad != []:
            display.EnsureVisible(self.bad[-1])

        # show the currently selected item
        try:
            display.Select(self.currentItem, True)
        except TypeError:#may need AttributeError
            pass
        else:
            display.EnsureVisible(self.currentItem)



##### RENAMING: ################################################################
    # show renaming errors
    def showRenameError(self, i, err, original, renamed):
        err = err.decode(sys.getfilesystemencoding())
        # set item as bad
        self.toRename[i][1][1] = None
        self.bottomWindow.display.EnsureVisible(i)
        self.bottomWindow.display.Refresh()
        # show error message
        self.setStatusMsg(err, u'failed')
        utils.makeErrMsg(_(u"Error renaming:\n%s \n\nto:\n%s \n\nReason: %s")\
                         %(original[0],renamed[0],err),
                         _(u"Renaming Failed !!"))


    # renaming operation, with error handling
    # returns error state
    def rename(self, i, original, renamed, refresh_int):
        display = self.bottomWindow.display
        try:
            shutil1.move(original[0], renamed[0], True)
        except IOError, (errNumb, err):
            # may need to create dirs first
            if errNumb == 2 and not os.path.exists(os.path.dirname(renamed[0])):
                try:
                    os.makedirs(os.path.dirname(renamed[0]))
                except OSError, (n, err):
                    self.showRenameError(i, err, original, renamed)
                    return 'makedirs'
                else:
                    self.rename(i, original, renamed, refresh_int)
            else:
                self.showRenameError(i, err, original, renamed)
                return True
        except OSError, (errNumb, err):
            self.showRenameError(i, err, original, renamed)
            return True
        # set item as renamed
        self.toRename[i][1][1] = True
        if i % refresh_int == 0:
            display.EnsureVisible(i)
            display.Update()

        return False


    # Write undo files first (safety first !), then attemp to perform
    # the renaming operation.
    def renameItems(self, event):

        def printList(list):
                for item in list:
                    print item
        
        itemsRenamed = 0
        error = False
        warn = False
        self.currentItem = None
        self.bottomWindow.display.mode = 'rename'
        sep = os.path.sep
        display = self.bottomWindow.display
        utils.setBusy(True)
        self.picker.Panel.path.SetEditable(False)

        if event != u'undo':
            path = self.picker.params.root

        # adjust and sort items when recursively renaming folders
        if self.recursiveFolderOn and event != u'undo':
            self.setStatusMsg(_(u"Adjusting %s recursive paths, please wait ...")%len(self.toRename),u'wait')

            progressDialog = utils.ProgressDialog(self, self.prefs, self.items,
                             _(u"Adjusting %%% recursive paths, please wait ..."))

            if self.showTimes:
                t = time.time()

            # define these here for faster processing
            def osSplit(item):
                return os.path.split(item)
            def addSep(path):
                return os.sep + path + os.sep
            # order by path depth
            def recursiveFolderSort(x, y):
                x = x[1][0].count(os.sep)
                y = y[1][0].count(os.sep)
                return cmp(x, y)

            self.toRename.sort(recursiveFolderSort)
            sortedRename = self.toRename
            

            # make a list of all folders that will be processed
            foldersToAdjust = []
            for item in sortedRename:
                # only grab folders
                if not item[0][1]:
                    #oF = osSplit(item[0][0])[0] # original folder name
                    #nF = osSplit(item[1][0])[0] # new folder name
                    oF = item[0][0]
                    nF = item[1][0]
                    if oF is not nF:
                        print oF, nF
                        #print item[1][0].count(os.sep)
                        foldersToAdjust.append((oF, nF))

            sys.exit()


            # replace all occurences of folders in path
            for i in range(len(self.toRename)):

                print foldersToAdjust
                for f in foldersToAdjust:
                    oF = addSep(f[0]) # original folder name
                    
                    # continue from this point for faster processing
                    if oF not in self.toRename[i][0][0]:
                        continue

                    nF = addSep(f[1]) # new folder name

                    print oF, nF
                    #print testing(i,oF,nF)

                    print self.toRename[i][0][0]
                    print self.toRename[i][0][0].replace(oF,nF,1)
                    print

                    
                    
                    self.toRename[i][0][0] = self.toRename[i][0][0].replace(oF,nF,1)
                    self.toRename[i][1][0] = self.toRename[i][1][0].replace(oF,nF,1)

                #if not progressDialog.update(i):
                #    error = 'cancelled'
                #    break

            # make sure renaming will be in correct order !
            self.toRename.sort(recursiveFolderSort)

            progressDialog.destroy()

            if self.showTimes:
                print "%s items recursive adjust : %s"%(len(self.toRename), (time.time() - t))


            

            #printList(foldersToAdjust)
            #printList(self.toRename)
            sys.exit()



        if not error:
            self.setStatusMsg(_(u"Renaming in progress, please wait ..."),u'wait')

            # open undo files for writing
            try:
                originalFile = codecs.open(utils.stdPath(u'undo/original.bak'),'w',"utf-8")
                renamedFile = codecs.open(utils.stdPath(u'undo/renamed.bak'),'w',"utf-8")
            except IOError, (errno, strerror):
                dlg = wx.MessageDialog(self, strerror +\
                      _(u"\nMake sure 'undo' directory exists and is read/write\n\nYou will not be able to undo!!\nDo you want to continue??"),
                      _(u"Problem with undo!"), wx.ICON_ERROR|wx.YES_NO)
                if dlg.ShowModal() == wx.ID_NO:
                    error = 'cancelled'
                dlg.Destroy()

        if not error:
            if self.showTimes:
                t = time.time()

            # dupe checking, windows is case insensitive
            if platform.system() == 'Windows':
                def compare(renamed, original):
                    if renamed[0].lower() != original[0].lower():
                        return True
                    else:
                        return False
            else:
                def compare(renamed, original):
                    if renamed[0] != original[0]:
                        return True
                    else:
                        return False

            # define here for faster processing
            def exists(item):
                return os.path.exists(item)
            def join(a,b,c):
                return unicode(os.path.join(a,b,c))

            # calculate refresh settings
            count = len(self.toRename)
            count = count / 25 + 1
            if count > int(self.prefs['renRefreshMin']):
                count = int(self.prefs['renRefreshMin'])

            # renaming operation:
            i = 0
            for original, renamed in self.toRename:
                if warn != 'duplicate_name' and compare(renamed, original) and\
                  exists(renamed[0]):
                    # set item as bad
                    self.toRename[i][1][1] = None
                    display.EnsureVisible(i)
                    display.Refresh()
                    self.setStatusMsg(_(u"Duplicate name"), u'warn')
                    msg = _(u"This name already exists:\n%s")%renamed[0]\
                        +_(u"\n\nI can make a sub-folder called 't_e_m_p' and do the renaming operation there.\nYou can then move everything back.\n\nGo ahead and do this?")
                    dlg = wx.MessageDialog(self, msg,_(u"Duplicate name"),
                        wx.YES_NO | wx.YES_DEFAULT | wx.ICON_WARNING)
                    if dlg.ShowModal() == wx.ID_YES:
                        warn = 'duplicate_name'
                    else:
                        error = True
                        break
                    dlg.Destroy()

                # set correct path if in dupe error mode
                if warn == 'duplicate_name':
                    if not path.endswith(sep):
                        path = path+sep
                    renamed[0] = renamed[0].replace(path,u'')
                    renamed[0] = join(path, u't_e_m_p', renamed[0])

                # write name to undo
                originalFile.write(original[0] + u'\n')
                renamedFile.write(renamed[0] + u'\n')

                error = self.rename(i, original, renamed, count)
                #os.renames(original[0], renamed[0])
                if error:
                    break

                i += 1

                if self.debug:
                    print original[0]
                    print renamed[0]
                    print


        # end of operations
        itemsRenamed = i

        if not error:
            self.setStatusMsg(_(u"Renaming for %s items completed")
                              %itemsRenamed,u"complete")
            if self.prefs['reloadAfterRename'] and event != u'undo':
                self.picker.refresh(event)
            else:
                self.picker.clearALL()
            if self.prefs[u'alwaysMakeLog']:
                self.saveAsText(False)
        elif error == 'cancelled':
            self.setStatusMsg(_(u"Renaming cancelled"),u'failed')
        else:
            self.setStatusMsg(_(u"Error Renaming"),u'failed')

        utils.setBusy(False)
        display.EnsureVisible(len(self.toRename)-1)
        display.Refresh()

        if self.showTimes:
            print "%s items renamed : %s"%(itemsRenamed, (time.time() - t))

        # auto
        if self.autoModeLevel > 2 and not error:
            print _(u"Renaming for %s items completed")%itemsRenamed
            sys.exit()

        self.picker.refreshDirTree()

        self.bottomWindow.go.Enable(False)
        self.menuFile.GoMenu.Enable(False)
        if event != 'undo':
            self.bottomWindow.setUndoRedoType('undo')
        self.picker.Panel.path.SetEditable(True)

#EOF
