# -*- coding: utf-8 -*-

# This is the panel in charge of directory manipulations.

"""
Copyright (C) 2006-2009 ianaré sévi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ianare@gmail.com
"""

import wx
import directoryTools
import numbering
import dateTime
import utils
import os.path

[wxID_PANEL, wxID_PANELNOTEBOOK,
] = [wx.NewId() for _init_ctrls in range(2)]

class OpPanel(wx.Panel):
    def sizer(self,parent):
        #smallestSize = parent.rightSizer.GetSize() - parent.rightTopSizer.GetSize() - (10,10)
        superSizer = wx.BoxSizer(wx.VERTICAL)
        #superSizer.SetMinSize(smallestSize)
        superSizer.Add(self.notebook,0,wx.EXPAND)
        self.SetSizerAndFit(superSizer)

    def _init_notebook(self, parent):
        parent.AddPage(imageId=-1, page=self.directoryToolsPanel, select=True,
              text=_(u"Directory settings"))
        parent.AddPage(imageId=-1, page=self.numberingPanel, select=False,
              text=_(u"Numbering settings"))
        parent.AddPage(imageId=-1, page=self.dateTimePanel, select=False,
              text=_(u"Date / Time settings"))
        # load and assign notebook page images
        il = wx.ImageList(16, 16)
        img0 = il.Add(wx.Bitmap(utils.iconPath(u'directory.ico'), wx.BITMAP_TYPE_ICO))
        img1 = il.Add(wx.Bitmap(utils.iconPath(u'numbering.ico'), wx.BITMAP_TYPE_ICO))
        img2 = il.Add(wx.Bitmap(utils.iconPath(u'date_time.ico'), wx.BITMAP_TYPE_ICO))
        parent.AssignImageList(il)
        parent.SetPageImage(0, img0)
        parent.SetPageImage(1, img1)
        parent.SetPageImage(2, img2)
        # required to make XP theme better:
        self.notebook.SetBackgroundColour(main.notebook.GetThemeBackgroundColour())


    def _init_ctrls(self, prnt):
        wx.Panel.__init__(self, id=wxID_PANEL, name=u'Panel', parent=prnt,
              style=wx.TAB_TRAVERSAL)

        self.notebook = wx.Notebook(id=wxID_PANELNOTEBOOK,
              name=u'notebook', parent=self, style=wx.NB_BOTTOM|wx.NO_BORDER)

        self.directoryToolsPanel = directoryTools.Panel(self.notebook, main)
        self.numberingPanel = numbering.Panel(self.notebook, main)
        self.dateTimePanel = dateTime.Panel(self.notebook, main)


    def __init__(self, parent, main_window):
        global main
        main = main_window
        self._init_ctrls(parent)
        self._init_notebook(self.notebook)
        self.dirTools = self.notebook.GetPage(0)
        self.sizer(parent)

    # update GUI elements, settings after config load
    def onConfigLoad(self):
        self.numberingPanel.onConfigLoad()
        self.dateTimePanel.getFromItemCheckbox(False)

    # extra operation for absolute paths
    def addPathStructure(self, newPath, path):
        recurPath = ''
        recur = self.dirTools.pathRecur.GetValue()
        path = path.split(os.sep)
        # remove drive letter
        if wx.Platform == '__WXMSW__':
            path = path[1:]

        # inverse the selection or not
        if not self.dirTools.inverse.GetValue():
            if recur <= 0:
                recur -= 1
            path = path[-recur:]
        else:
        	path = path[:-recur]

        # reassemble path
        for segment in path:
            recurPath = os.path.join(recurPath,segment)
        newPath = newPath.replace(self.directoryToolsPanel.pathStructTxt,recurPath)
        return newPath

    # used to reset the numbering counter for the operation
    def resetCounter(self, c):
        utils.resetCounter(self, self.directoryToolsPanel, c)

    # create the new path
    def renameItem(self, path, name, ext, original):
        rejoin = False
        operations = self.directoryToolsPanel.opButtonsPanel
        newPath = self.dirTools.directoryText.GetValue()
        pathStructTxt = self.directoryToolsPanel.pathStructTxt

        # absolute path
        if os.path.isabs(newPath):
            split = os.path.splitdrive(newPath)
            newPath = split[1]
            rejoin = True

        # add path structure
        if pathStructTxt in newPath:
            newPath = self.addPathStructure(newPath,path)

        newPath = operations.parseInput(newPath,original,self)

        if rejoin:
            newPath = os.path.join(split[0],newPath)
        else:
            newPath = os.path.join(path,newPath)

        return (newPath,name,ext)




