# -*- coding: utf-8 -*-

# This panel controls insertAtg.

"""
Copyright (C) 2006-2009 ianaré sévi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ianare@gmail.com
"""

import wx
import insertTools
import numbering
import dateTime
import utils

[wxID_PANEL, wxID_PANELNOTEBOOK] = [wx.NewId() for _init_ctrls in range(2)]

class OpPanel(wx.Panel):
    def sizer(self, parent):
        #smallestSize = parent.rightSizer.GetSize() - parent.rightTopSizer.GetSize() - (10,10)
        mainSizer = wx.BoxSizer(wx.VERTICAL)
        #mainSizer.SetMinSize(smallestSize)
        mainSizer.Add(self.notebook,0,wx.EXPAND)
        self.SetSizerAndFit(mainSizer)

    def _init_notebook(self, parent):
        # generated method, don't edit
        parent.AddPage(imageId=-1, page=self.InsertToolsPanel, select=True,
              text=_(u"Insert settings"))
        parent.AddPage(imageId=-1, page=self.numberingPanel, select=False,
              text=_(u"Numbering settings"))
        parent.AddPage(imageId=-1, page=self.dateTimePanel, select=False,
              text=_(u"Date / Time settings"))
        # load and assign notebook page images
        il = wx.ImageList(16, 16)
        img0 = il.Add(wx.Bitmap(utils.iconPath(u'insert.ico'), wx.BITMAP_TYPE_ICO))
        img1 = il.Add(wx.Bitmap(utils.iconPath(u'numbering.ico'), wx.BITMAP_TYPE_ICO))
        img2 = il.Add(wx.Bitmap(utils.iconPath(u'date_time.ico'), wx.BITMAP_TYPE_ICO))
        parent.AssignImageList(il)
        parent.SetPageImage(0, img0)
        parent.SetPageImage(1, img1)
        parent.SetPageImage(2, img2)
        # required to make XP theme better:
        self.notebook.SetBackgroundColour(main.notebook.GetThemeBackgroundColour())

    def _init_ctrls(self, prnt):
        wx.Panel.__init__(self, id=wxID_PANEL, name=u'Panel', parent=prnt,
              style=wx.TAB_TRAVERSAL)

        self.notebook = wx.Notebook(id=wxID_PANELNOTEBOOK,
              name=u'notebook', parent=self, style=wx.NB_BOTTOM|wx.NO_BORDER)
        self.notebook.SetThemeEnabled(True)

        self.InsertToolsPanel = insertTools.Panel(self.notebook, main)

        self.numberingPanel = numbering.Panel(self.notebook, main)

        self.dateTimePanel = dateTime.Panel(self.notebook, main)


    def __init__(self, parent, main_window):
        global main
        main = main_window
        self._init_ctrls(parent)
        self._init_notebook(self.notebook)
        self.sizer(parent)

    # update GUI elements, settings after config load
    def onConfigLoad(self):
        self.InsertToolsPanel.activateOptions(False)
        self.numberingPanel.onConfigLoad()
        self.dateTimePanel.getFromItemCheckbox(False)

    # used to reset the numbering counter for the operation
    def resetCounter(self, c):
        utils.resetCounter(self, self.InsertToolsPanel, c)

    # insert into name:
    def renameItem(self, path, name, ext, original):
        newName = utils.joinExt(self.opsApply,name,ext)
        if not newName:
            return path,name,ext

        insert = self.InsertToolsPanel
        operations = insert.opButtonsPanel
        parsedText = operations.parseInput(insert.Text.GetValue(),original,self)
        # prefix
        if insert.prefix.GetValue():
            newName = parsedText + name
        # suffix
        elif insert.suffix.GetValue():
            newName = name + parsedText
        # exact position
        elif insert.position.GetValue():
                pos = insert.positionPos.GetValue()
                if pos == -1:
                    newName += parsedText
                elif pos < -1:
                    newName = newName[:pos+1] + parsedText + newName[pos+1:]
                else:
                    newName = newName[:pos] + parsedText + newName[pos:]

        # insert before/after a character:
        elif insert.after.GetValue() or insert.before.GetValue():
            good2go = False
            textMatch = insert.BAtextMatch.GetValue()
            # text search
            if not insert.regExpPanel.reg_expr.GetValue():
                try:
                    insertAt = newName.index(textMatch)
                except ValueError:
                    pass
                else:
                    if insert.after.GetValue():
                        insertAt += len(textMatch)
                    good2go = True
            # regular expression search
            else:
                insertRE = insert.regExpPanel.createRE(textMatch)
                try:
                    insertAt = insertRE.search(newName).end()
                except AttributeError:
                    pass
                else:
                    if insert.before.GetValue():
                        insertAt -= len(textMatch)
                    good2go = True

            if good2go:
                newName = newName[:insertAt] + parsedText + newName[insertAt:]

        # insert in between 2 characters:
        # (copy of search.py function)
        elif insert.between.GetValue():
            good2go = False
            if insert.regExpPanel.reg_expr.GetValue():
                mod1 = insert.regExpPanel.createRE(insert.BTWtextMatch1.GetValue())
                mod2 = insert.regExpPanel.createRE(insert.BTWtextMatch2.GetValue())
                try:
                    frm = mod1.search(newName).end()
                    to = mod2.search(newName, frm).start()
                except AttributeError:
                    pass
                else:
                    good2go = True
            else:
                match1 = insert.BTWtextMatch1.GetValue()
                match2 = insert.BTWtextMatch2.GetValue()
                try:
                    frm = newName.index(match1)+len(match1)
                    to = newName.index(match2,frm)
                except ValueError:
                    pass
                else:
                    good2go = True
            if good2go:
                newName = newName[:frm] + parsedText + newName[to:]


        name,ext = utils.splitExt(self.opsApply,newName,name,ext)

        return path,name,ext
