# -*- coding: utf-8 -*-

# this is the renamer panel, it allows displaying and controlling
# all operation panels.

"""
Copyright (C) 2006-2009 ianaré sévi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ianare@gmail.com
"""

import wx
import operations
import re
import utils

[wxID_MAINPANEL, wxID_MAINPANELAPPLYEXTENSION, wxID_MAINPANELAPPLYNAME,
 wxID_MAINPANELAVAILABLEOPERATIONS, wxID_MAINPANELDELETEOPERATION,
 wxID_MAINPANELENABLEOPERATION, wxID_MAINPANELMOVEDOWN, wxID_MAINPANELMOVEUP,
 wxID_MAINPANELSTATICTEXT1, wxID_MAINPANELSTATICTEXT2,
 wxID_MAINPANELSTATICTEXT3, wxID_MAINPANELSTATICTEXT4,
 wxID_MAINPANELUSEDOPERATIONS, wxID_MAINPANELSTATICTEXT5,
 wxID_MAINPANELMENUUP, wxID_MAINPANELMENUDOWN, wxID_MAINPANELDELETEOPERATIONS,
 wxID_MAINPANELMENURESET, wxID_MAINPANELMENUSETSTATUS,
 wxID_MAINPANELMENUDISABLE, wxID_MAINPANELMENUAPPLYNAME,
 wxID_MAINPANELMENUAPPLYEXTENSION, wxID_MAINPANELMENUDESTROY,
 wxID_MAINPANELMENUDESTROYALL, wxID_MAINPANELRESETOPERATIONBUTTON,
] = [wx.NewId() for _init_ctrls in range(25)]


BackgroundClr = wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW)
HighlightClr = wx.SystemSettings_GetColour(wx.SYS_COLOUR_HIGHLIGHT)
HighlightTxtClr = wx.SystemSettings_GetColour(wx.SYS_COLOUR_HIGHLIGHTTEXT)
TxtClr = wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOWTEXT)

class OperationDropTarget(wx.TextDropTarget):
    def __init__(self, object):
        wx.TextDropTarget.__init__(self)
        self.object = object
        self.prnt = object.GetParent()
        self.itemCount = 0

    def HitTest(self, x, y):
        return self.prnt.usedOperations.HitTest(wx.Point(x,y))


    # set all items to deselected
    def ResetHighlight(self, last=True):
        #print self.itemCount
        for pos in range(self.itemCount):
            item = wx.ListItem()
            item.SetId(pos)
            item.SetBackgroundColour(BackgroundClr)
            item.SetTextColour(TxtClr)
            self.prnt.usedOperations.SetItem(item)
            if not last:
                self.prnt.usedOperations.SetItemState(pos,0,wx.LIST_STATE_SELECTED)

    def OnLeave(self):
        self.itemCount = self.prnt.usedOperations.GetItemCount()
        self.ResetHighlight()
        self.prnt.usedOperations.Refresh()
        if self.curSelected is not None:
            self.prnt.usedOperations.Select(self.curSelected)

    def OnEnter(self, x, y, d):
        self.itemCount = self.prnt.usedOperations.GetItemCount()
        self.curSelected = self.prnt.getCurrentOperationNumber()
        return d

    def OnDragOver(self, x, y, d):
        #print "OnDragOver: %d, %d, %d" % (x, y, d)
        pos = self.HitTest(x,y)[0]
        if pos != -1:
            self.ResetHighlight(False)
            item = wx.ListItem()
            item.SetId(pos)
            item.SetBackgroundColour(HighlightClr)
            item.SetTextColour(HighlightTxtClr)
            item.SetState(wx.LIST_STATE_FOCUSED)
            self.prnt.usedOperations.SetItem(item)
            self.prnt.usedOperations.Refresh()
        return d

    def OnDropText(self, x, y, data):
        pos = self.HitTest(x,y)[0]
        self.prnt.addOperationToStack(data,pos)
        self.ResetHighlight()


# Show intro text, set app size
class IntroTextPanel(wx.Panel):
    def __init__(self,prnt,ID):
        wx.Panel.__init__(self, id=ID, name=u'IntroTextPanel',
              parent=prnt, size=wx.DefaultSize, style=wx.TAB_TRAVERSAL)

        # create info text of available operations
        # Find largest panel, use it to set this panel size
        txt = ''
        txtSize = (0,0)
        for i in range(prnt.availableOperations.GetItemCount()):
            op = prnt.availableOperations.GetItemText(i)
            txt += "%s  :  %s\n\n"%(op, operations.defs[op][1])

            # create temporary instance of panel to get size
            op = operations.defs[op][0]
            opPanel = getattr(operations, op).OpPanel(prnt, main)
            size = opPanel.GetSizeTuple()
            if size > txtSize:
                txtSize = size
            opPanel.Destroy()

        # adjust vertical spacing for top bar
        gap = prnt.moveDown.GetSizeTuple()[1]+8

        self.staticText1 = wx.StaticText(id=-1,
              label=_(u"You don't have any operations defined."),
              name='staticText1', parent=self, style=wx.ALIGN_LEFT,
              size=wx.Size(txtSize[0],-1))
        self.staticText1.SetFont(wx.Font(12, wx.SWISS, wx.NORMAL, wx.BOLD))

        self.staticText2 = wx.StaticText(id=-1,
              label=_("Double-click or Drag/Drop an operation in the 'Available' box to add one."),
              name='staticText2', parent=self, style=wx.ALIGN_LEFT,
              size=wx.Size(txtSize[0],-1))
        self.staticText2.SetFont(wx.Font(10, wx.SWISS, wx.NORMAL, wx.BOLD))

        self.staticText3 = wx.StaticText(id=-1,
              label=txt, name='staticText3', parent=self,
              style=wx.ALIGN_LEFT, size=wx.Size(txtSize[0],-1))

        # add to sizer
        mainSizer = wx.BoxSizer(wx.VERTICAL)
        mainSizer.Add(self.staticText1,0,wx.BOTTOM,8)
        mainSizer.Add(self.staticText2,0,wx.BOTTOM,30)
        mainSizer.Add(self.staticText3,0)
        self.SetSizer(mainSizer)

        # set size based on calculations above
        self.SetMinSize(wx.Size(txtSize[0],txtSize[1]+gap))



class Panel(wx.Panel):
    def sizers(self):
        mainSizer = self.mainSizer = wx.BoxSizer(wx.HORIZONTAL)

        leftSizer = wx.BoxSizer(wx.VERTICAL)
        leftSizer.Add(self.staticText1,0,)
        leftSizer.Add(self.availableOperations,0,wx.BOTTOM,5)
        leftSizer.Add(self.staticText2,0,wx.TOP,5)
        leftSizer.Add(self.usedOperations,1)
        mainSizer.Add(leftSizer,0,wx.EXPAND|wx.ALL, 5)

        rightSizer = self.rightSizer = wx.BoxSizer(wx.VERTICAL)
        rightTopSizer = self.rightTopSizer = wx.BoxSizer(wx.HORIZONTAL)
        rightTopSizer.Add(self.staticText3,0,wx.ALIGN_CENTRE|wx.LEFT,5)
        rightTopSizer.Add(self.moveDown,0,wx.ALIGN_CENTRE|wx.LEFT,3)
        rightTopSizer.Add(self.moveUp,0,wx.ALIGN_CENTRE|wx.RIGHT,25)
        rightTopSizer.Add((-1,-1),1)
        rightTopSizer.Add(self.staticText4,0,wx.ALIGN_CENTRE|wx.RIGHT,5)
        rightTopSizer.Add(self.applyName,0,wx.ALIGN_CENTRE|wx.RIGHT,3)
        rightTopSizer.Add(self.applyExtension,0,wx.ALIGN_CENTRE|wx.RIGHT,25)
        rightTopSizer.Add((-1,-1),5)
        rightTopSizer.Add(self.enableOperation,0,wx.ALIGN_CENTRE|wx.RIGHT,13)
        rightTopSizer.Add(self.resetOperationButton,0,wx.ALIGN_CENTRE|wx.RIGHT,13)
        rightTopSizer.Add(self.deleteOperations,0,wx.ALIGN_CENTRE|wx.RIGHT,5)
        rightTopSizer.Add((-1,-1),5)
        rightSizer.Add(rightTopSizer,0,wx.TOP|wx.BOTTOM|wx.EXPAND,3)
        rightSizer.Add(self.staticText5,0,wx.TOP|wx.LEFT,20)

        mainSizer.Add(rightSizer,1,wx.EXPAND)
        self.SetSizerAndFit(mainSizer)

    # The used operations' right click menu
    def _init_menu_items(self, parent, n):
        # 'apply to' sub menu
        parent.applyTo = applyTo = wx.Menu()
        applyTo.name = wx.MenuItem(applyTo, wxID_MAINPANELMENUAPPLYNAME,
            _(u"Name"), kind = wx.ITEM_CHECK)
        applyTo.extension = wx.MenuItem(applyTo, wxID_MAINPANELMENUAPPLYEXTENSION,
            _(u"Extension"), kind = wx.ITEM_CHECK)

        applyTo.AppendItem(applyTo.name)
        applyTo.AppendItem(applyTo.extension)

        self.Bind(wx.EVT_MENU, self.setOperationsApplyMenu,
              id=wxID_MAINPANELMENUAPPLYNAME)
        self.Bind(wx.EVT_MENU, self.setOperationsApplyMenu,
              id=wxID_MAINPANELMENUAPPLYEXTENSION)

        opPanel = self.Core.operations[n]

        if opPanel.opsApply[0]:
            applyTo.name.Check(True)
        if opPanel.opsApply[1]:
            applyTo.extension.Check(True)

        # main menu
        parent.up = wx.MenuItem(parent, wxID_MAINPANELMENUUP, _(u"Move up"))
        parent.up.SetBitmap(wx.Bitmap(utils.iconPath(u'up.png'),
            wx.BITMAP_TYPE_PNG))

        parent.down = wx.MenuItem(parent, wxID_MAINPANELMENUDOWN, _(u"Move down"))
        parent.down.SetBitmap(wx.Bitmap(utils.iconPath(u'down.png'),
            wx.BITMAP_TYPE_PNG))

        if opPanel.IsEnabled():
            txt = _(u"Disable")
            ico = 'disable'
        else:
            txt = _(u"Enable")
            ico = 'enable'
        parent.disable = wx.MenuItem(parent, wxID_MAINPANELMENUDISABLE, txt)
        parent.disable.SetBitmap(wx.Bitmap(utils.realPath(u"icons/%s.png"%ico),
            wx.BITMAP_TYPE_PNG))

        parent.reset = wx.MenuItem(parent, wxID_MAINPANELMENURESET, _(u"Reset"))
        parent.reset.SetBitmap(wx.Bitmap(utils.iconPath(u'reset_op.png'),
            wx.BITMAP_TYPE_PNG))

        parent.destroy = wx.MenuItem(parent, wxID_MAINPANELMENUDESTROY, _(u"Destroy"))
        parent.destroy.SetBitmap(wx.Bitmap(utils.iconPath(u'errors.ico'),
            wx.BITMAP_TYPE_ICO))

        parent.destroyAll = wx.MenuItem(parent, wxID_MAINPANELMENUDESTROYALL, _(u"Destroy All"))
        parent.destroyAll.SetBitmap(wx.Bitmap(utils.iconPath(u'nuke.png'),
            wx.BITMAP_TYPE_PNG))

        parent.setStatus = wx.MenuItem(parent, wxID_MAINPANELMENUSETSTATUS, _(u"Enable/Disable"))
        parent.setStatus.SetBitmap(wx.Bitmap(utils.iconPath(u're.ico'),
            wx.BITMAP_TYPE_ICO))

        parent.AppendItem(parent.up)
        parent.AppendItem(parent.down)
        if _(u"directory") not in self.usedOperations.GetItemText(n):
            parent.applyToMenu = wx.MenuItem(parent, -1, _(u"Apply to"),
                subMenu = applyTo)
            parent.AppendItem(parent.applyToMenu)
        parent.AppendItem(parent.disable)
        parent.AppendItem(parent.reset)
        parent.AppendItem(parent.destroy)
        parent.AppendItem(parent.destroyAll)

        self.Bind(wx.EVT_MENU, self.onMoveUpButton,
              id=wxID_MAINPANELMENUUP)
        self.Bind(wx.EVT_MENU, self.onMoveDownButton,
              id=wxID_MAINPANELMENUDOWN)
        self.Bind(wx.EVT_MENU, self.onOperationTogglebutton,
              id=wxID_MAINPANELMENUDISABLE)
        self.Bind(wx.EVT_MENU, self.resetOperation,
              id=wxID_MAINPANELMENURESET)
        self.Bind(wx.EVT_MENU, self.deleteOperation,
              id=wxID_MAINPANELMENUDESTROY)
        self.Bind(wx.EVT_MENU, self.destroyAllGUIOperations,
              id=wxID_MAINPANELMENUDESTROYALL)


    def _init_ctrls(self, prnt):
        wx.Panel.__init__(self, id=wxID_MAINPANEL, name=u'mainPanel',
              parent=prnt, size=wx.DefaultSize, style=wx.TAB_TRAVERSAL)

        self.staticText1 = wx.StaticText(id=wxID_MAINPANELSTATICTEXT1,
              label=_(u"Available:"), name='staticText1', parent=self,
              style=0)

        self.availableOperations = wx.ListCtrl(id=wxID_MAINPANELAVAILABLEOPERATIONS,
              name=u'availableOperations', parent=self,
              style=wx.LC_REPORT | wx.LC_NO_HEADER | wx.LC_SINGLE_SEL)
        self.availableOperations.Bind(wx.EVT_LIST_ITEM_ACTIVATED,
              self.addOperationToStack, id=wxID_MAINPANELAVAILABLEOPERATIONS)

        self.availableOperations.InsertColumn(0, '', format=wx.LIST_FORMAT_LEFT, width=-1)

        # add operations to list
        i = 0
        for op in sorted(operations.defs.keys()):
            self.availableOperations.InsertStringItem(i, op)
            i += 1

        # set size
        h = self.availableOperations.GetItemRect(0)[-1]
        ops = self.availableOperations.GetItemCount()

        if wx.Platform == '__WXMSW__':
            h = (h*ops)+6
        else:
            h = h*ops
        w = 132
        sbSize = 10

        Size = (w,h)
        self.availableOperations.SetMinSize(Size)
        self.availableOperations.SetColumnWidth(0,w-sbSize)

        self.usedOperations = wx.ListCtrl(id=wxID_MAINPANELUSEDOPERATIONS,
              name=u'usedOperations', parent=self, size=wx.Size(w, -1),
              style=wx.LC_REPORT | wx.LC_NO_HEADER | wx.LC_SINGLE_SEL)
        self.usedOperations.Bind(wx.EVT_LIST_ITEM_SELECTED,
            self.onUsedOperationsListbox, id=wxID_MAINPANELUSEDOPERATIONS)

        self.usedOperations.InsertColumn(0, '', format=wx.LIST_FORMAT_LEFT,
                                         width=w-sbSize)

        self.staticText2 = wx.StaticText(id=wxID_MAINPANELSTATICTEXT2,
              label=_(u"Used:"), name='staticText2', parent=self,
              style=0)

        self.moveDown = wx.BitmapButton(bitmap=wx.Bitmap(utils.iconPath(u'down.png'),
              wx.BITMAP_TYPE_PNG), id=wxID_MAINPANELMOVEDOWN, name=u'moveDown',
              parent=self, style=wx.BU_AUTODRAW)
        self.moveDown.SetToolTipString(_(u"Move current operation down by 1"))
        self.moveDown.Bind(wx.EVT_BUTTON, self.onMoveDownButton,
              id=wxID_MAINPANELMOVEDOWN)

        self.moveUp = wx.BitmapButton(bitmap=wx.Bitmap(utils.iconPath(u'up.png'),
              wx.BITMAP_TYPE_PNG), id=wxID_MAINPANELMOVEUP, name=u'moveUp',
              parent=self, style=wx.BU_AUTODRAW)
        self.moveUp.SetToolTipString(_(u"Move current operation up by 1"))
        self.moveUp.Bind(wx.EVT_BUTTON, self.onMoveUpButton,
              id=wxID_MAINPANELMOVEUP)

        self.enableOperation = wx.ToggleButton(id=wxID_MAINPANELENABLEOPERATION,
              label=_(u"Disable"), name=u'enableOperation', parent=self,
              style=wx.BU_EXACTFIT)
        self.enableOperation.SetToolTipString(_(u"Enable or Disable current operation"))
        self.enableOperation.SetValue(False)
        self.enableOperation.Bind(wx.EVT_TOGGLEBUTTON,
              self.onOperationTogglebutton, id=wxID_MAINPANELENABLEOPERATION)

        self.deleteOperations = wx.Choice(choices=[_(u"Destroy"), _(u"Destroy All")],
              id=wxID_MAINPANELDELETEOPERATIONS, name=u'actions', parent=self)
        self.deleteOperations.SetSelection(0)
        self.deleteOperations.SetToolTipString(_(u"Do Stuff"))
        self.deleteOperations.Bind(wx.EVT_CHOICE, self.onActionsChoice, id=wxID_MAINPANELDELETEOPERATIONS)

        self.resetOperationButton = wx.Button(id=wxID_MAINPANELRESETOPERATIONBUTTON,
              label=_(u"Reset"), name=u'resetOperationButton', parent=self,
              style=wx.BU_EXACTFIT)
        self.resetOperationButton.SetToolTipString(_(u"Reset the current operation"))
        self.resetOperationButton.Bind(wx.EVT_BUTTON, self.resetOperation,
              id=wxID_MAINPANELRESETOPERATIONBUTTON)

        self.staticText3 = wx.StaticText(id=wxID_MAINPANELSTATICTEXT3,
              label=_(u"Order:"), name='staticText3', parent=self,
              style=0)

        self.applyExtension = wx.CheckBox(id=wxID_MAINPANELAPPLYEXTENSION,
              label=_(u"extension"), name=u'applyExtension', parent=self,
              style=0)
        self.applyExtension.SetValue(False)
        self.applyExtension.Bind(wx.EVT_CHECKBOX, self.setOperationsApply,
              id=wxID_MAINPANELAPPLYEXTENSION)

        self.applyName = wx.CheckBox(id=wxID_MAINPANELAPPLYNAME,
              label=_(u"name"), name='applyName', parent=self, style=0)
        self.applyName.SetValue(True)
        self.applyName.Bind(wx.EVT_CHECKBOX, self.setOperationsApply,
              id=wxID_MAINPANELAPPLYNAME)

        self.staticText4 = wx.StaticText(id=wxID_MAINPANELSTATICTEXT4,
              label=_(u"Apply to:"), name='staticText4', parent=self,
              style=0)

        self.staticText5 = IntroTextPanel(self, wxID_MAINPANELSTATICTEXT5)

        dt = OperationDropTarget(self.usedOperations)
        self.usedOperations.SetDropTarget(dt)

        wx.EVT_LIST_BEGIN_DRAG(self, self.availableOperations.GetId(),
            self.onDragInit)


    def __init__(self, Core, parent, main_window):
        global main
        main = main_window
        self.Core = Core

        self._init_ctrls(parent)
        self.sizers()
        self.setButtonState()
        # capture right clicks in several areas
        for area in (self, self.usedOperations):
            # for wxMSW
            area.Bind(wx.EVT_COMMAND_RIGHT_CLICK, self.onRightClick)
            # for wxGTK
            area.Bind(wx.EVT_RIGHT_UP, self.onRightClick)


    # currently selected operation number
    def getCurrentOperationNumber(self):
        usedOps = self.usedOperations
        if usedOps.GetSelectedItemCount() > 0:
            return usedOps.GetNextItem(-1, state=wx.LIST_STATE_SELECTED)
        else:
            return False

    # currently selected operation panel
    def getCurrentOperationPanel(self):
        n = self.getCurrentOperationNumber()
        return self.Core.operations[n]


    # Popup the menu
    def onRightClick(self, event):
        self.operationsMenu = wx.Menu()
        n = self.getCurrentOperationNumber()
        if n is not False:
            self._init_menu_items(self.operationsMenu, n)
            self.PopupMenu(self.operationsMenu)


    # Start dragging from available operations
    def onDragInit(self, event):
        text = self.availableOperations.GetItemText(event.GetIndex())
        tdo = wx.PyTextDataObject(text)
        tds = wx.DropSource(self.availableOperations)
        tds.SetData(tdo)
        tds.DoDragDrop(True)

    def addOperationToSizer(self, opPanel):
        self.rightSizer.Detach(self.staticText5)
        self.staticText5.Show(False)
        self.rightSizer.Add(opPanel,1,wx.EXPAND|wx.RIGHT,5)
        self.mainSizer.Layout()

    def addOperationToList(self, pos, opName):
        self.usedOperations.InsertStringItem(pos,unicode(pos+1)+u": "+ opName)

    def addOperationToStack(self, event, pos=-1):
        if type(event) == type(u''):
            op = event
            #event = False
        else:
            op = event.GetLabel()

        # adding operation
        if pos == -1:
            pos = self.usedOperations.GetItemCount()
        # loading operation from config
        elif type(pos) == type(u' '):
            pos = int(pos)
        # inserting operation
        else:
            pos += 1

        # add requested operation
        self.addOperationToList(pos, op)
        opPanel = self.createOperation(pos, op)
        self.reassignNumbers()

        # add window to sizer:
        self.addOperationToSizer(opPanel)

        self.usedOperations.Select(pos)
        self.setButtonState()
        main.showPreview(event)


    def createOperation(self, n, op):
        # create the requested window:
        operation = getattr(operations, operations.defs[op][0])
        opPanel = operation.OpPanel(self, main)
        self.Core.insertOperation(n, opPanel)

        # directory change operations:
        if op == _(u"directory"):
            self.Core.operations[n].opsApply = [False,False]
            self.hideShowApplyChkBx(False)
        # all others:
        else:
            self.Core.operations[n].opsApply = [True,False]
            self.hideShowApplyChkBx(True)
        return opPanel


    # set 'apply to' from menu
    def setOperationsApplyMenu(self, event):
        id = event.GetId()
        if id == wxID_MAINPANELMENUAPPLYNAME:
            checked = self.operationsMenu.applyTo.name.IsChecked()
            self.applyName.SetValue(checked)
        else:
            checked = self.operationsMenu.applyTo.extension.IsChecked()
            self.applyExtension.SetValue(checked)
        self.setOperationsApply(event)

    # set to which part of name operation applies to:
    def setOperationsApply(self,event):
        operation = self.getCurrentOperationPanel()
        if operation is not False:
            applyName = self.applyName.GetValue()
            applyExtension = self.applyExtension.GetValue()
            operation.opsApply = [applyName, applyExtension]
            main.showPreview(event)


    # enable/disable apply_to checkboxes
    def hideShowApplyChkBx(self, show):
        chkbx = (self.applyName, self.applyExtension)
        if show:
            for box in chkbx:
                box.Enable(True)
            self.applyName.SetValue(True)
            self.applyExtension.SetValue(False)
        else:
            for box in chkbx:
                box.SetValue(False)
                box.Enable(False)


    # Enable/disable delete and enable buttons
    def setButtonState(self):
        opButtons = (self.enableOperation, self.resetOperationButton,
                     self.deleteOperations,
                     self.moveUp, self.moveDown)
        if len(self.Core.operations) < 1:
            for i in opButtons:
                i.Enable(False)
            for i in (self.applyName, self.applyExtension):
                i.Enable(False)
        else:
            for i in opButtons:
                i.Enable(True)


    # Show a specific operation
    def showOperation(self, n):
        # need to hide other operations first
        for opPanel in self.Core.operations:
            opPanel.Show(False)
            self.rightSizer.Detach(opPanel)

        # now show selected one
        opPanel = self.Core.operations[n]
        opPanel.Show(True)
        self.addOperationToSizer(opPanel)

        # disable name/ext checkboxes if directory operation:
        opsApply = opPanel.opsApply

        if _(u"directory") not in self.usedOperations.GetItemText(n):
            # show if name and/or extension is operated on:
            self.applyName.SetValue(opsApply[0])
            self.applyExtension.SetValue(opsApply[1])

            self.applyName.Enable(True)
            self.applyExtension.Enable(True)
        else:
            self.hideShowApplyChkBx(False)

        # show operation button as enabled/disabled
        if opPanel.IsEnabled():
            self.enableOperation.SetLabel(_(u"Disable"))
            self.enableOperation.SetValue(False)
        else:
            self.enableOperation.SetLabel(_(u"Enable"))
            self.enableOperation.SetValue(True)
        self.mainSizer.Layout()


    # show operation from box
    def onUsedOperationsListbox(self, event):
        n = self.getCurrentOperationNumber()
        if n is not False:
            self.showOperation(n)


    # Enable or disable the currently selected operation:
    def onOperationTogglebutton(self, event):
        opPanel = self.getCurrentOperationPanel()
        if opPanel is not False:
            if opPanel.IsEnabled():
                opPanel.Enable(False)
                self.enableOperation.SetLabel(_(u"Enable"))
            else:
                opPanel.Enable(True)
                self.enableOperation.SetLabel(_(u"Disable"))
            self.mainSizer.Layout()
            main.showPreview(event)


    # Delete an operation, then reasign numbers to remaining ones
    def deleteOperation(self, event):
        n = self.getCurrentOperationNumber()
        if n is not False:
            type = self.usedOperations.GetItemText(n)
            type = re.sub("\d{1,}: ", '', type)

            dlg = wx.MessageDialog(self, _("Really destroy this '%s' operation?")%type,
                _("Are You Sure?"), wx.YES_NO|wx.YES_DEFAULT|wx.ICON_WARNING)
            if dlg.ShowModal() == wx.ID_YES:
                # delete operation:
                self.usedOperations.DeleteItem(n)
                self.Core.deleteOperation(n)
                self.reassignNumbers()

                # select another operation, if present
                if len(self.Core.operations) > 0:
                    # there is a preceding operation
                    if n-1 > -1:
                        n = n-1
                    self.showOperation(n)
                    self.usedOperations.Select(n)
                # otherwise show text
                else:
                    self.staticText5.Show(True)
                self.setButtonState()
                main.showPreview(event)

    # called by deleteOperation
    def reassignNumbers(self):
        for i in range(self.usedOperations.GetItemCount()):
            original = self.usedOperations.GetItemText(i)
            self.usedOperations.SetItemText(i,str(i+1)+original[1:])


    # Destroy, then recreate operation (reset)
    def resetOperation(self, event):
        n = self.getCurrentOperationNumber()
        if n is not False:
            text = self.usedOperations.GetItemText(n)
            text = re.search("(?<=: ).+$", text)
            op = text.group()
            dlg = wx.MessageDialog(self, _("Really reset this '%s' operation?")%op,
                _("Are You Sure?"), wx.YES_NO|wx.YES_DEFAULT|wx.ICON_WARNING)
            if dlg.ShowModal() == wx.ID_YES:
                # delete operation:
                self.Core.deleteOperation(n)

                # create new of same type/position
                opPanel = self.createOperation(n, op)

                # add it to the window properly
                self.addOperationToSizer(opPanel)
                main.showPreview(event)

    # destroy all operations in stack
    def destroyAllOperations(self):
        self.Core.destroyAllOperations()
        self.usedOperations.DeleteAllItems()
        self.staticText5.Show(True)
        self.setButtonState()

    # Destroy all operations from GUI
    def destroyAllGUIOperations(self, event):
        dlg = wx.MessageDialog(self, _("Really destroy all operations?"),
                _("Are You Sure?"), wx.YES_NO|wx.YES_DEFAULT|wx.ICON_WARNING)
        if dlg.ShowModal() == wx.ID_YES:
            self.destroyAllOperations()
            main.showPreview(event)

    # execute correct function based on user selected action
    def onActionsChoice(self, event):
        selected = self.deleteOperations.GetSelection()
        result = {0 : self.deleteOperation,
                  1 : self.destroyAllGUIOperations,
                 }[selected](event)
        self.deleteOperations.SetSelection(0)


    # CHANGE THE ORDER OF OPERATIONS
    def moveOperations(self, To):
        n = self.getCurrentOperationNumber()
        if n is not False:
            moveTo = n + To
            if self.usedOperations.GetItemCount() > moveTo and moveTo > -1:
                oldText = self.usedOperations.GetItemText(n)
                self.usedOperations.DeleteItem(n)
                self.usedOperations.InsertStringItem(moveTo,oldText)
                self.reassignNumbers()
                
                self.Core.moveOperations(n, moveTo)
                self.usedOperations.Select(moveTo)
            main.showPreview(True)

    def onMoveDownButton(self, event):
        self.moveOperations(+1)

    def onMoveUpButton(self, event):
        self.moveOperations(-1)

