# -*- coding: utf-8 -*-

# Helping functions

"""
Copyright (C) 2006-2009 ianaré sévi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ianare@gmail.com
"""

import wx
import wx.html
import os
import sys
import codecs
import locale
import time
try:
    import Image
except:
    pass


# Color definitions (get from system)
try:
    BackgroundClr = wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW)
    HighlightClr = wx.SystemSettings_GetColour(wx.SYS_COLOUR_HIGHLIGHT)
    HighlightTxtClr = wx.SystemSettings_GetColour(wx.SYS_COLOUR_HIGHLIGHTTEXT)
    TxtClr = wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOWTEXT)
except:
    pass


# get version info from file
def getVersion():
    try:
        f = open(realPath("version"))
    except:
        v = 'unknown'
    else:
        v = f.readline().strip()
        f.close()
    return v + u' (beta)'


# get wxPython Version
def getWxVersion():
    wxVer = ""
    for x in wx.VERSION:
        wxVer = wxVer + str(x) + "."
    wxVer = wxVer.rstrip(".")
    return wxVer
                

# determine if PIL is loaded
def pilIsLoaded():
    try:
        Image
    except NameError:
        return False
    else:
        return True
            
                            
# reset all sizes to platform defaults (modified from Boa-Constructor)
def resetMinSize(parent, ignoreCtrls=(), ignoreClasses=()):
    # need this for some cases in linux, otherwise cuts off text
    if wx.Platform == '__WXGTK__' and isinstance(parent, wx.StaticText):
        textSize = parent.GetTextExtent(parent.GetLabel())
        size = wx.Size(textSize[0]+2,-1)
    else:
        size = wx.DefaultSize

    parent.SetMinSize(size)
    parent.SetSize(size)

    for child in parent.GetChildren():
        if child not in ignoreCtrls and not isinstance(child, ignoreClasses):
            resetMinSize(child, ignoreCtrls, ignoreClasses)


# adjust button sizes when using BU_EXACTFIT
# ignore is a tuple of buttons to ignore
def adjustExactbuttons(parent, ignore=()):
    for child in parent.GetChildren():
        if isinstance(child, wx.Button) and child.GetName() not in ignore:
            txt = child.GetLabel()
            size = wx.Size(child.GetTextExtent(txt)[0] + 18,-1)
            child.SetMinSize(size)


# create necessary folders
def makeInitialEnvironment():
    if not os.path.exists(stdPath('undo')):
        try:
            os.makedirs(stdPath('undo'))
        except OSError, error:
            if error[0] == 17:
                pass
            else:
                makeErrMsg(unicode(error), u"Error")
                pass


# Calculate the correct start and end positions for slices
def calcStartEndPos(s,l):
    if s == -1 and l == 1:
        frm = -1
        to = None
    elif s < 0 and l > 1:
        frm = s#-l+1
        to = s+l
        if to >= 0:
            to = None
    else:
        frm = s
        to = s+l
    return frm,to


# Generic error message
def makeErrMsg(msg, title):
    dlg = wx.MessageDialog(None, msg, title, wx.ICON_ERROR)
    dlg.ShowModal()
    dlg.Destroy()


# return user path
def stdPath(file):
    base = wx.StandardPaths.Get().GetUserConfigDir()
    return os.path.join(base,u'.metamorphose2',file)


# return application path
def realPath(file):
    if hasattr(sys, "frozen"):
        path = os.path.dirname(sys.executable)
    else:
        path = False
        for path in sys.path:
            if 'metamorphose' in path:
                path = path.decode(sys.getfilesystemencoding())
                break
        if not path:
            print "Could not determine application path.\nMake sure the application is correctly installed.\n"
            sys.exit();        
        #print path
    return os.path.join(path,file)

# return locale directory
def localePath(lang):
    # windows py2exe
    if hasattr(sys, "frozen"):
        return realPath(u'messages')
    # linux, freeBSD when installed
    elif os.path.exists(u'/usr/share/locale/%s/LC_MESSAGES/metamorphose2.mo'%lang):
        return u'/usr/share/locale'
    # run from source
    else:
        return realPath(u'../messages')

# return full icon path
def iconPath(icon):
    return realPath(u'icons/%s'%icon)

# add item to warnings
def addToWarnings(main,path,msg):
    ec = main.ec
    if main.warn.count(ec) < 1:
        main.warn.append(ec)
        main.errorLog.insert(0, (ec, path, msg, u'warn'))

# add item to errors
def addToErrors(main,path,msg):
    ec = main.ec
    if main.bad.count(ec) < 1:
        main.bad.append(ec)
        main.errorLog.insert(0, (ec, path, msg, u'bad'))


# reset a numbering counter
def resetCounter(parent, Tools, c):
    Tools.opButtonsPanel.counter = c
    Tools.opButtonsPanel.auxCount = 0
    parent.numberingPanel.setNumberParams(True)


# write a file
def writeFile(path, data):
    try:
        os.makedirs(os.path.dirname(path))
    except:
        pass
    try:
        fp = codecs.open(path, 'w', 'utf_8')
    except (IOError, OSError), error:
        makeErrMsg(unicode(error), _(u"Error"))
        pass
    else:
        fp.write(unicode(data))
        fp.close()


# split extensions (for operations)
def joinExt(opsApply, name, ext):
    if opsApply[0] and not opsApply[1]:
        newName = name
    elif opsApply[1] and not opsApply[0]:
        newName = ext
    elif opsApply[0] and opsApply[1]:
        if ext != '':
            newName = name+'.'+ext
        else:
            newName = name
    else:
        newName = False
    return newName

# join extensions (for operations)
def splitExt(opsApply, newName, name, ext):
    if opsApply[0] and not opsApply[1]:
        name = newName
    elif opsApply[1] and not opsApply[0]:
        ext = newName
    elif opsApply[0] and opsApply[1]:
        name = os.path.splitext(newName)[0]
        ext = os.path.splitext(newName)[1][1:]
    return name,ext

# get all notebook page names
def getNotebookPages(parent):
    pages = []
    for i in range(parent.notebook.GetPageCount()):
        pages.append(parent.notebook.GetPage(i))
    return pages

# sometimes need to bind to this when programmatically doing things
def devNull(event):
    pass

# Time encode/decode crap, a workaround for strftime not accepting unicode:
def uDate(main, format, itemDateTime):
    udate = time.strftime(format.encode(main.encoding),itemDateTime)
    return udate.decode(main.encoding)

def setBusy(busy):
    if busy:
        wx.BeginBusyCursor()
    else:
        wx.EndBusyCursor()

# encodings lookup
def getEncodingChoices(key):
    encodingLookup = {
        _(u"Use System Default") : [u"%s"%locale.getlocale()[1], ],
        _(u'All Languages / Unicode') : [u'utf_8', u'utf_16'],
        _(u'Western Europe') : [u'windows-1252', u'iso-8859-1', u'iso-8859-15',
            u'mac_roman', u'cp500', u'cp850', u'cp1140'],
        _(u'Central & Eastern Europe') : [u'windows-1250', u'iso-8859-2',
            u'mac_latin2', u'cp852'],
        _(u'Esperanto, Maltese') : [u'iso-8859-3', ],
        _(u'Nordic Languages') : [u'iso-8859-10', u'mac_iceland', u'cp861',
            u'cp865'],
        _(u'Celtic Languages') : [u'iso8859_14', ],
        _(u'Baltic Languages') : [u'windows-1257',u'iso-8859-13',u'cp775'],
        _(u'Cyrillic Languages') : [u'windows-1251',u'iso-8859-5',u'koi8_r',
            'koi8_u',u'mac_cyrillic',u'cp154',u'cp866',u'cp855'],
        _(u'Greek') : [u'windows-1253',u'iso-8859-7',u'mac_greek',u'cp737',
            u'cp869',u'cp875'],
        _(u'Turkish') : [u'windows-1254', u'iso-8859-9', u'mac_turkish',
            u'cp857', u'cp1026'],
        _(u'Hebrew') : [u'windows-1255', u'iso-8859-8', u'cp424', u'cp856',
            u'cp862'],
        _(u'Arabic') : [u'windows-1256', u'iso-8859-6', u'cp864'],
        _(u'Urdu') : [u'cp1006', ],
        _(u'Thai') : [u'cp874', ],
        _(u'Vietnamese') : [u'windows-1258', ],
        _(u'Traditional Chinese') : [u'ms950', u'big5hkscs', u'big5'],
        _(u'Simplified Chinese') : [u'gb2312', u'hz'],
        _(u'Unified Chinese') : [u'ms936', u'gb18030'],
        _(u'Korean') : [u'ms949', u'iso-2022-kr', u'ms1361', u'euc_kr'],
        _(u'Japanese') : [u'ms-kanji', u'iso-2022-jp', u'shift_jis', u'euc_jp',
            u'iso-2022-jp-1', u'iso-2022-jp-2004', u'iso-2022-jp-3',
            u'iso-2022-jp-ext', u'shift_jis_2004', u'shift_jisx0213',
            u'euc_jis_2004', u'euc_jisx0213'],
        }
    return encodingLookup[key]



#### TINY CLASSES ####

# base class to load parameters from GUI
class Parameters:
    def setValueMethod(self):
        self.getValue = self.getWxValue
        
    # get a wxWidget value
    def getWxValue(self, widget):
        if isinstance(widget, wx.ComboBox):
            value = widget.GetStringSelection()
            # inconsistences in library requires this
            if not value:
                value = widget.GetValue()
        elif isinstance(widget, wx.Choice):
            value = widget.GetSelection()
        else:
            value = widget.GetValue()
        return value
    
    # set given widgets
    def setParameters(self, widgets):
        for widgetName in widgets:
            if hasattr(self.Panel, widgetName):
                widget = getattr(self.Panel, widgetName)
                setattr(self, widgetName, self.getValue(widget))
        return self
    
    
# display single page help dialog
class SmallHelp(wx.Dialog):
    def _init_ctrls(self, prnt, main, Title, Icon):
        # generated method, don't edit
        wx.Dialog.__init__(self, id=-1, name='smallHelpDialog', parent=prnt,
              size=wx.Size(610, 531),
              style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER | wx.MAXIMIZE_BOX, title=Title)
        self.SetIcon(wx.Icon(realPath(u'icons/%s.ico')%Icon,wx.BITMAP_TYPE_ICO))

        self.display = wx.html.HtmlWindow(id=-1,
              name=u'display', parent=self, size=wx.Size(602,
              520), style=wx.html.HW_SCROLLBAR_AUTO)

    def __init__(self, parent, main, helpFile, Title, Icon):
        self._init_ctrls(parent,main,Title,Icon)
        self.Center(wx.HORIZONTAL|wx.VERTICAL)
        if u'gtk2' in wx.PlatformInfo:
            self.display.SetStandardFonts()

        docspath = realPath(u'help')
        
        if os.path.isfile(os.path.join(docspath,main.language,helpFile)):
            helpFile = os.path.join(docspath,main.language,helpFile)
        else:
            helpFile = os.path.join(docspath,u'en_US',helpFile)

        self.display.LoadPage(helpFile)


# progress dialog handling
class ProgressDialog(wx.ProgressDialog):
    def __init__(self, parent, prefs, items, message):
        if type(items) is not type(1):
            items = len(items)
        message = message.replace(u'%%%', unicode(items))
        
        if prefs['showProgressDialog'] and items > int(prefs[u'itemCountForProgDialog']):
            self.active = True
            wx.ProgressDialog.__init__(self,
                                       _(u"Progress"),
                                       message,
                                       maximum = items,
                                       parent=parent,
                                       style =  wx.PD_APP_MODAL
                                        | wx.PD_ELAPSED_TIME
                                        |wx.PD_AUTO_HIDE
                                        | wx.PD_CAN_ABORT
                                       )
        else:
            self.active = False

    def update(self, c):
        if self.active:
            (keepGoing, skip) = self.Update(c)
            return keepGoing
    
    def destroy(self):
        if self.active:
            self.Destroy()
            self = False

#EOF
